@@ -30,7 +30,7 @@ local do_trace = false
 
 local local_networks = interface.getNetworksStats()
 
-for _, net_stats in ipairs(local_networks) do
+for _, net_stats in pairs(local_networks) do
    network.select(net_stats.network_id)
    
    checks.localNetworkChecks(granularity, checks_var, do_trace) 
