@@ -1,5 +1,5 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 --
@@ -8,12 +8,14 @@
 
 local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
+package.path = dirs.installdir .. "/scripts/lua/modules/vulnerability_scan/?.lua;" .. package.path
+
 
 require "lua_utils"
 
 -- Important: load this before any other alert related module
-local plugins_utils = require "plugins_utils"
-plugins_utils.loadPlugins()
+local script_manager = require "script_manager"
+script_manager.loadScripts()
 
 local recipients = require "recipients"
 recipients.initialize()
@@ -29,6 +31,7 @@ local delete_data_utils = require "delete_data_utils"
 local ts_utils = require "ts_utils"
 local presets_utils = require "presets_utils"
 local blog_utils = require("blog_utils")
+local vs_utils = require "vs_utils"
 
 -- ##################################################################
 
@@ -58,7 +61,6 @@ if ntop.isAppliance() then
    ntop.reloadPreferences()
 end
 
-host_pools_nedge.migrateHostPools()
 if ntop.isnEdge() then
    host_pools_nedge.initPools()
 end
@@ -83,16 +85,15 @@ local function cleanupIfname(ifname, ifid)
    local alerts_status_path = os_utils.fixPath(dirs.workingdir .. "/" .. ifid .. "/json/")
    ntop.rmdir(alerts_status_path)
 
-   -- Remove the active devices and pools keys
-   alert_utils.deleteActiveDevicesKey(ifid)
-   alert_utils.deleteActivePoolsKey(ifid)
-
    -- Remove network discovery request on startup
    discover_utils.clearNetworkDiscovery(ifid)
 
    -- Clean old InfluxDB export cache
    local export_dir = os_utils.fixPath(dirs.workingdir .. "/".. ifid .."/ts_export")
    ntop.rmdir(export_dir)
+
+   -- Clean redis queue used to push host filters to pfring
+   ntop.delCache("pfring." .. ifname .. ".filter.host.queue")
 end
 
 -- Remove the json dumps previously needed for alerts generation
@@ -121,6 +122,8 @@ lists_utils.startup()
 
 -- ##################################################################
 
+traceError(TRACE_NORMAL, TRACE_CONSOLE, "Initializing device polices...")
+
 -- Initialize device policies (presets)
 -- NOTE: Must go after lists_utils initialization and reload
 -- as new custom protocols can be set by lists utils
@@ -130,11 +133,12 @@ presets_utils.reloadAllDevicePolicies()
 -- TODO: migrate custom re-arm settings
 
 -- this will retrieve host pools and policers configurtions via HTTP if enabled
-if  ntop.isnEdge() then
+if ntop.isnEdge() then
    local http_bridge_conf_utils = require "http_bridge_conf_utils"
    http_bridge_conf_utils.configureBridge()
 end
 
+traceError(TRACE_NORMAL, TRACE_CONSOLE, "Initializing alerts...")
 alert_utils.notify_ntopng_start()
 
 if not recovery_utils.check_clean_shutdown() then
@@ -143,6 +147,7 @@ end
 
 recovery_utils.unmark_clean_shutdown()
 
+traceError(TRACE_NORMAL, TRACE_CONSOLE, "Initializing timeseries...")
 -- Need to run setup at startup since the schemas may be changed
 ts_utils.setupAgain()
 
@@ -172,6 +177,12 @@ for ifid, ifname in pairs(delete_data_utils.list_all_interfaces()) do
    ntop.delCache("ntopng.dhcp."..ifid..".cache")
 end
 
+-- Remove notification cache
+local notifications = ntop.getKeysCache("ntopng.cache.alerts.notification.*") or {}
+for k, _ in pairs(notifications) do
+  ntop.delCache(k)
+end
+
 if(has_pcap_dump_interface) then
   -- Load the lists at the very beginning in order to avoid misclassification
   -- when reading from PCAP dump. This can take some time.
@@ -191,12 +202,41 @@ if(ntop.exists(local_startup_file)) then
 end
 
 if(ntop.isPro()) then
-   -- Import ClickHouse dumps if any
-   local silence_import_warnings = true
-
-   ntop.importClickHouseDumps(silence_import_warnings)
+   if ntop.isClickHouseEnabled() then
+      -- Import ClickHouse dumps if any
+      local silence_import_warnings = true
+      traceError(TRACE_NORMAL, TRACE_CONSOLE, "Importing ClickHouse dumps...")
+      ntop.importClickHouseDumps(silence_import_warnings)
+   end
 end
 
-blog_utils.fetchLatestPosts()
+-- Fetch latest ntop blog posts
+if not ntop.isnEdge() then
+  -- Note: On nEdge they are fetched in a dayly/delayed callback as connectivity
+  -- may be not yet up at this stage
+  blog_utils.fetchLatestPosts()
+end
+
+
+-- Cleanup old influxdb files (if any)
+local influxdb_dir = dirs.workingdir .. "/tmp/influxdb"
+if(ntop.exists(influxdb_dir)) then
+   local files = ntop.readdir(influxdb_dir)
+
+   if(files ~= nil) then
+      for _, name in pairs(files) do
+	 if (ends(name, ".tmp") == true) then
+	    local fname = influxdb_dir .. "/" .. name
+
+	    -- io.write("[InfluxDB] Deleting file "..fname.."\n")
+	    ntop.unlink(fname)
+	 end
+      end
+   end
+end
+
+-- Vulnerability scan activities
+vs_utils.migrate_keys()
+vs_utils.restore_host_to_scan()
 
 traceError(TRACE_NORMAL, TRACE_CONSOLE, "Completed startup.lua")
