@@ -110,7 +110,7 @@ for list_name, list in pairs(lists) do
   list.status_label = getListStatusLabel(list)
 
   if sortColumn == "column_category_name" then
-    sort_to_key[list_name] = getCategoryLabel(list.category_name)
+    sort_to_key[list_name] = getCategoryLabel(list.category_name, list.category)
   elseif sortColumn == "column_last_update" then
     sort_to_key[list_name] = list.status.last_update
   elseif sortColumn == "column_num_hosts" then
@@ -119,6 +119,8 @@ for list_name, list in pairs(lists) do
     sort_to_key[list_name] = list.status_label
   elseif sortColumn == "column_update_interval_label" then
     sort_to_key[list_name] = list.update_interval
+  elseif sortColumn == "column_num_hits" then
+    sort_to_key[list_name] = list.status.num_hits or 0
   else
     -- default
     sort_to_key[list_name] = list_name
@@ -154,10 +156,11 @@ for key in pairsByValues(sort_to_key, sOrder) do
       column_status = list.status_label,
       column_url = list.url,
       column_enabled = list.enabled,
+      column_num_hits = ternary(list.status.num_hits > 0, format_utils.formatValue(list.status.num_hits), ''),
       column_update_interval = list.update_interval,
       column_update_interval_label = update_interval_label,
       column_category = "cat_" .. list.category,
-      column_category_name = getCategoryLabel(list.category_name),
+      column_category_name = getCategoryLabel(list.category_name, list.category),
       column_num_hosts = ternary(list.status.num_hosts > 0, format_utils.formatValue(list.status.num_hosts), ''),
       column_last_update = format_utils.formatPastEpochShort(list.status.last_update),
     }
