@@ -1,5 +1,5 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -14,15 +14,46 @@ if not ntop.isnEdge() then
    return
 end
 
-local remote_addr = _SERVER["REMOTE_ADDR"]
-local is_logged = captive_portal_utils.is_logged(remote_addr)
+local info = ntop.getInfo()
 
-if is_logged then
-   captive_portal_utils.logout(remote_addr)
+local remote_addr = _SERVER["REMOTE_ADDR"]
+local method = _SERVER["REQUEST_METHOD"]
+
+local is_logged = captive_portal_utils.is_logged(remote_addr)
+if method == "POST" then
+   if is_logged then
+      captive_portal_utils.logout(remote_addr)
+      is_logged = false
+   end
 end
 
 captive_portal_utils.print_header()
 
+if is_logged then
+  print [[
+   <form role="form" class="form-signin" action="]] print(ntop.getHttpPrefix()) print[[/lua/captive_portal.lua" method="POST">
+	 <h2 class="form-signin-heading" style="font-weight: bold;">]] print(info["product"]) print [[ Access Portal</h2>
+    <div class="form-group mb-3 has-feedback">
+       <input type="hidden" class="form-control" name="action" value="logout" />
+       <input type="hidden" class="form-control" name="csrf" value="]] print(ntop.getRandomCSRFValue()) print [[" />
+        <small>
+          <p>]] print(i18n("login.already_logged")) print(" ") print(i18n("login.logout_message")) print [[
+        </small>
+    </div>
+    <button class="w-100 btn btn-lg btn-primary" type="submit">]] print(i18n("login.logout")) print[[</button>
+  	<div class="row">
+      <div >&nbsp;</div>
+      <div class="col-lg-12">
+        <small>
+          <p>]] print(info["copyright"]) print [[
+        </small>
+      </div>
+    </div>
+  </form>
+]]
+
+else
+
 print [[
 	 <form id="form_add_user" role="form" data-bs-toggle="validator" class="form-signin" onsubmit="return makeUsernameLowercase();" action="]] print(ntop.getHttpPrefix()) print[[/lua/authorize_captive.lua]]
 
@@ -45,13 +76,10 @@ if not isEmptyString(r) then
    print("?referer="..r)
 end
 
-local info = ntop.getInfo()
-
 print[[" method="POST">
 	 <h2 class="form-signin-heading" style="font-weight: bold;">]] print(info["product"]) print [[ Access Portal</h2>
   <div class="form-group mb-3 has-feedback">
-      <input type="hidden" class="form-control" name="username">
-      <input type="text" class="form-control" name="_username" placeholder="]] print(i18n("login.username")) print[[" pattern="^[\w\.%]{1,}$" required>
+      <input type="text" class="form-control" name="username" placeholder="]] print(i18n("login.username")) print[[" pattern="^[\w\.%]{1,}$" required>
       <input type="password" class="form-control" name="password" placeholder="]] print(i18n("login.password")) print[[" pattern="]] print(getPasswordInputPattern()) print[[" required>
       <input type="text" class="form-control" name="label" placeholder="]] print(i18n("login.device_label")) print[[" pattern="^[ \w\.%]{1,}$" required>
 </div>
@@ -60,11 +88,8 @@ print[[" method="POST">
       <div >&nbsp;</div>
       <div class="col-lg-12">
         <small>
-      <p>]] print(i18n("login.enter_credentials")) print[[
-          </p>
-
-      <p>]] print(info["copyright"]) print [[
-
+          <p>]] print(i18n("login.enter_credentials")) print[[</p>
+          <p>]] print(info["copyright"]) print [[</p>
         </small>
       </div>
     </div>
@@ -76,13 +101,12 @@ print[[" method="POST">
 
   function makeUsernameLowercase() {
     var target = $('#form_add_user input[name="username"]');
-    var origin = $('#form_add_user input[name="_username"]');
-    target.val(origin.val().toLowerCase());
-    origin.removeAttr("name");
+    target.val(target.val().toLowerCase());
     return true;
   }
 </script>
 
 ]]
+end
 
 captive_portal_utils.print_footer()
