@@ -1,5 +1,5 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -10,7 +10,7 @@ local page_utils = require("page_utils")
 
 sendHTTPContentTypeHeader('text/html')
 
-page_utils.set_active_menu_entry(page_utils.menu_entries.checks_dev)
+page_utils.print_header_and_set_active_menu_entry(page_utils.menu_entries.checks_dev)
 
 -- #######################################################
 
