@@ -13,7 +13,7 @@ local ui_utils = require("ui_utils")
 
 sendHTTPContentTypeHeader('text/html')
 
-page_utils.set_active_menu_entry(page_utils.menu_entries.geo_map)
+page_utils.print_header_and_set_active_menu_entry(page_utils.menu_entries.geo_map)
 
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
