@@ -5,12 +5,12 @@
 local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/alert_store/?.lua;" .. package.path
 
+local checks = require("checks")
+local other_alert_keys = require "other_alert_keys"
+
 local am_utils = {}
 local ts_utils = require "ts_utils_core"
-local format_utils = require "format_utils"
 local json = require("dkjson")
-local plugins_utils = require("plugins_utils")
-local os_utils = require("os_utils")
 local alerts_api = require("alerts_api")
 local alert_consts = require("alert_consts")
 local lua_path_utils = require("lua_path_utils")
@@ -28,6 +28,14 @@ local HOUR_STATS_UNREACHABLE = 3
 
 local do_trace = false
 
+local active_monitoring = {
+   -- Script category
+   category = checks.check_categories.active_monitoring,
+
+   default_enabled = true,
+   alert_id = other_alert_keys.alert_am_threshold_cross,
+}
+
 -- ##############################################
 
 local am_hosts_key = string.format("ntopng.prefs.ifid_%d.am_hosts", getSystemInterfaceId())
@@ -243,7 +251,9 @@ local function key2amhost(host)
   local measurement, amhost = string.match(host, "^([^@]+)@(.+)")
 
   if measurement and amhost then
-     return amhost, measurement
+    return amhost, measurement
+  else
+    return nil, nil
   end
 end
 
@@ -386,6 +396,7 @@ local function deserializeAmPrefs(host_key, val, config_only)
     local v = json.decode(val)
 
     if v then
+      rv.host_key = host_key
       rv.show_iface = false
       rv.ifname = v.ifname or ''
       rv.threshold = tonumber(v.threshold) or 500
@@ -417,7 +428,7 @@ end
 function am_utils.getHosts(config_only, granularity)
   local hosts = ntop.getHashAllCache(am_hosts_key) or {}
   local rv = {}
-
+  
   for host_key, val in pairs(hosts) do
     local host = deserializeAmPrefs(host_key, val, config_only)
 
@@ -467,16 +478,13 @@ end
 -- @param measurement A string with the type of measurement which will be performed
 -- @param am_value A number used as threshold con consider the measurement failed
 -- @param granularity One of `supported_granularities`, indicating the granularity of the measurement
--- @param pool The pool_id `host` will be associated to
 -- @param token A string with an ntopng `token` used to fetch data from other ntopngs in a federation [optional]
 -- @param save_result Whether the result fetched with the measure should be saved (e.g., the HTTP response) [optional]
 -- @param readonly Bool used by the GUI to know if, when true, an entry is considered read only hence it cannot be modified/deleted [optional]
-function am_utils.addHost(host, ifname, measurement, am_value, granularity, pool, token, save_result, readonly)
+function am_utils.addHost(host, ifname, measurement, am_value, granularity, token, save_result, readonly)
   save_result = save_result or false
   readonly = readonly or false
 
-  local active_monitoring_pools = require("active_monitoring_pools")
-  local am_pool = active_monitoring_pools:create()
   local host_key = am_utils.getAmHostKey(host, measurement)
   local show_iface = ntop.isPingIfaceAvailable()
 
@@ -489,9 +497,6 @@ function am_utils.addHost(host, ifname, measurement, am_value, granularity, pool
     save_result = save_result, -- save the result
     readonly = readonly,
   }))
-
-  -- Bind the host from any existing pool
-  am_pool:bind_member(host_key, pool)
 end
 
 -- ##############################################
@@ -503,10 +508,8 @@ end
 -- ##############################################
 
 function am_utils.deleteHost(host, measurement)
-  local active_monitoring_pools = require("active_monitoring_pools")
-  local am_pool = active_monitoring_pools:create()
   local ts_utils = require("ts_utils")
-  local alert_utils = require("alert_utils")
+  -- local alert_utils = require("alert_utils")
 
   -- NOTE: system interface must be manually sected and then unselected
   local old_iface = tostring(interface.getId())
@@ -529,20 +532,17 @@ function am_utils.deleteHost(host, measurement)
 
   ntop.delHashCache(am_hosts_key, host_key)
 
-  -- Unbind the host from any existing pool
-  am_pool:bind_member(host_key, am_pool.DEFAULT_POOL_ID)
-
   -- Select the old interface
   interface.select(old_iface)
 end
 
 -- ##############################################
 
-local loaded_am_plugins = {}
+local loaded_am_scripts = {}
 local loaded_measurements = {}
 
-local function loadAmPlugins()
-   if not table.empty(loaded_am_plugins) then
+local function loadAmScripts()
+   if not table.empty(loaded_am_scripts) then
       return
    end
 
@@ -555,30 +555,30 @@ local function loadAmPlugins()
       end
 
       local mod_fname = string.sub(fname, 1, string.len(fname) - 4)
-      local plugin = require(mod_fname)
+      local script = require(mod_fname)
 
-      if not plugin then
+      if not script then
 	 traceError(TRACE_ERROR, TRACE_CONSOLE, string.format("Could not load '%s'", mod_fname))
 	 package.loaded[mod_fname] = nil
 	 goto continue
       end
 
-      if not plugin.measurements then
+      if not script.measurements then
 	 traceError(TRACE_ERROR, TRACE_CONSOLE, string.format("'measurements' section missing in '%s'", mod_fname))
 	 package.loaded[mod_fname] = nil
 	 goto continue
       end
 
-      if plugin.setup then
-	 -- A setup function exists, call it to determine if the plugin is available
-	 if(plugin.setup() == false) then
+      if script.setup then
+	 -- A setup function exists, call it to determine if the script is available
+	 if(script.setup() == false) then
 	    package.loaded[mod_fname] = nil
 	    goto continue
 	 end
       end
 
       -- Check that the measurements does not exist
-      for _, measurement in pairs(plugin.measurements) do
+      for _, measurement in pairs(script.measurements) do
 	 if(measurement.check == nil) then
 	    traceError(TRACE_ERROR, TRACE_CONSOLE, string.format("Missing 'check' function in '%s' measurement", measurement.key))
 	    goto skip
@@ -594,27 +594,31 @@ local function loadAmPlugins()
 	    goto skip
 	 end
 
-	 loaded_measurements[measurement.key] = {plugin=plugin, measurement=measurement}
+	 loaded_measurements[measurement.key] = {script=script, measurement=measurement}
 
 	 ::skip::
       end
 
-      plugin.key = mod_fname
-      loaded_am_plugins[mod_fname] = plugin
+      script.key = mod_fname
+      loaded_am_scripts[mod_fname] = script
 
       ::continue::
    end
+
+  loaded_measurements['vulnerability_scan'] = { measurement = { i18n_label = "active_monitoring_stats.vulnerability_scan" }}
+  loaded_measurements['ports_changes_detected'] = { measurement = { i18n_label = "active_monitoring_stats.ports_changes_detected" }}
+  loaded_measurements['cve_changes_detected'] = { measurement = { i18n_label = "active_monitoring_stats.cve_changes_detected" }}
 end
 
 -- ##############################################
 
 --! @brief Splits the hosts list by measurement.
 --! @param all_hosts the host list, whose format matches am_utils.getHosts()
---! @return a table measurement_key -> <plugin, measurement, hosts>
+--! @return a table measurement_key -> <script, measurement, hosts>
 function am_utils.getHostsByMeasurement(all_hosts)
   local hosts_by_measurement = {}
 
-  loadAmPlugins()
+  loadAmScripts()
 
   for key, host in pairs(all_hosts) do
     local measurement = host.measurement
@@ -626,7 +630,7 @@ function am_utils.getHostsByMeasurement(all_hosts)
       local measurement_key = m_info.measurement.key
 
       if not hosts_by_measurement[measurement_key] then
-	hosts_by_measurement[measurement_key] = {plugin = m_info.plugin, measurement = m_info.measurement, hosts = {}}
+	hosts_by_measurement[measurement_key] = {script = m_info.script, measurement = m_info.measurement, hosts = {}}
       end
 
       hosts_by_measurement[measurement_key].hosts[key] = host
@@ -638,12 +642,12 @@ end
 
 -- ##############################################
 
---! @brief Get a list of measurements from the loaded Active Monitoring plugins
+--! @brief Get a list of measurements from the loaded Active Monitoring scripts
 --! @return a list of measurements <title, value> for the gui.
 function am_utils.getAvailableMeasurements()
   local measurements = {}
 
-  loadAmPlugins()
+  loadAmScripts()
 
   for k, v in pairsByKeys(loaded_measurements, asc) do
     local m = v.measurement
@@ -662,7 +666,7 @@ end
 --! @brief Check if the specified measurement is available
 --! @return true if available, false otherwise
 function am_utils.isMeasurementAvailable(measurement)
-  loadAmPlugins()
+  loadAmScripts()
 
   return(loaded_measurements[measurement] ~= nil)
 end
@@ -675,7 +679,7 @@ end
 function am_utils.getAvailableGranularities(measurement)
   local granularities = {}
 
-  loadAmPlugins()
+  loadAmScripts()
 
   local m_info = loaded_measurements[measurement]
 
@@ -703,7 +707,7 @@ end
 --! @param measurement the measurement key
 --! @return the measurement metadata on success, nil on failure
 function am_utils.getMeasurementInfo(measurement)
-  loadAmPlugins()
+  loadAmScripts()
 
   local m_info = loaded_measurements[measurement]
 
@@ -719,7 +723,7 @@ end
 --! @brief Get the metadata of all the loaded measurements
 --! @return a list containing the measurements metadata
 function am_utils.getMeasurementsInfo()
-  loadAmPlugins()
+  loadAmScripts()
 
   local rv = {}
 
@@ -761,6 +765,7 @@ function am_utils.triggerAlert(numeric_ip, ip_label, current_value, upper_thresh
     -- Unreachable
     local host, measurement = key2amhost(ip_label)
     local info = am_utils.getMeasurementInfo(measurement)
+    type_info:set_score_critical()
 
     if info and info.unreachable_alert_i18n then
       -- The measurement provides an alternative message for the alert
@@ -838,7 +843,7 @@ end
 
 -- ##############################################
 
-function am_utils.editHost(host, iface, measurement, threshold, granularity, pool, token, save_result, readonly)
+function am_utils.editHost(host, iface, measurement, threshold, granularity, token, save_result, readonly)
   local existing = am_utils.getHost(host, measurement)
 
   if(existing == nil) then
@@ -876,7 +881,7 @@ function am_utils.editHost(host, iface, measurement, threshold, granularity, poo
     end
   end
 
-  am_utils.addHost(host, iface, measurement, threshold, granularity, pool, token, save_result, readonly)
+  am_utils.addHost(host, iface, measurement, threshold, granularity, token, save_result, readonly)
 
   return(true)
 end
@@ -936,10 +941,14 @@ function am_utils.run_am_check(when, all_hosts, granularity)
     local operator = info.measurement.operator
     local jitter = tonumber(info.jitter)
     local mean = tonumber(info.mean)
+    local calculate_scaling = true
 
+    if info.calculate_scaling then
+      calculate_scaling = info.calculate_scaling
+    end
     if(do_trace) then
        print("[AM result] "..key.."\n")
-       tprint(info)
+       -- tprint(info)
     end
 
     if jitter then jitter = round(jitter, 2) end
@@ -948,20 +957,21 @@ function am_utils.run_am_check(when, all_hosts, granularity)
     if areSystemTimeseriesEnabled() then
        local value = host_value
 
-       if info.measurement.chart_scaling_value then
+       if (calculate_scaling) and (info.measurement.chart_scaling_value) and (info.measurement.chart_scaling_value == 1) then
          value = value * info.measurement.chart_scaling_value
        end
 
        local ts_data = {ifid = getSystemInterfaceId(), host = host.host, metric = host.measurement, value = value}
        if(do_trace) then
 	  print("[Writing AM timeseries ")
-	  tprint(ts_data)
+	  -- tprint(ts_data)
        end
 
        ts_utils.append(am_schema, ts_data, when)
     end
 
     am_utils.setLastAmUpdate(key, when, host_value, resolved_host, jitter, mean)
+    alerts_api.setCheck(active_monitoring)
 
     if am_utils.hasExceededThreshold(threshold, operator, host_value) then
       if(do_trace) then print("[TRIGGER] Host "..resolved_host.."/"..key.." [value: "..host_value.."][threshold: "..threshold.."]\n") end
