@@ -3,11 +3,13 @@
 --
 
 local checks = require("checks")
+local alert_consts = require("alert_consts")
 local host_alert_keys = require "host_alert_keys"
 
 local domain_names_contacts = {
    -- Script category
    category = checks.check_categories.network,
+   severity = alert_consts.get_printable_severities().notice,
 
    default_enabled = false,
    alert_id = host_alert_keys.host_alert_domain_names_contacts,
@@ -21,7 +23,6 @@ local domain_names_contacts = {
 
    -- Allow user script configuration from the GUI
    gui = {
-      -- Localization strings, from the "locales" directory of the plugin
       i18n_title = "alerts_thresholds_config.domain_names_contacts_title",
       i18n_description = "alerts_thresholds_config.domain_names_contacts_description",
 
