@@ -4,10 +4,12 @@
 
 local checks = require("checks")
 local host_alert_keys = require "host_alert_keys"
+local alert_consts = require("alert_consts")
 
 local syn_scan = {
    -- Script category
    category = checks.check_categories.network,
+   severity = alert_consts.get_printable_severities().error,
 
    default_enabled = false,
    alert_id = host_alert_keys.host_alert_syn_scan,
@@ -21,7 +23,6 @@ local syn_scan = {
 
    -- Allow user script configuration from the GUI
    gui = {
-      -- Localization strings, from the "locales" directory of the plugin
       i18n_title = "entity_thresholds.syn_scan_title",
       i18n_description = "entity_thresholds.syn_scan_description",
 
