@@ -10,16 +10,16 @@ local os_utils = require("os_utils")
 local categories_utils = require("categories_utils")
 local json = require("dkjson")
 local alerts_api = require("alerts_api")
-local alert_severities = require "alert_severities"
 local alert_consts = require "alert_consts"
+local file_utils = require "file_utils"
 
 -- ##############################################
 
 -- NOTE: metadata and status are handled as separate keys.
 -- Metadata can only be updated by the gui, whereas status can only be
 -- updated by housekeeping. This avoid concurrency issues.
-local METADATA_KEY = "ntopng.prefs.category_lists.metadata"
-local STATUS_KEY = "ntopng.prefs.category_lists.status"
+local METADATA_KEY = "ntopng.cache.category_lists.metadata"
+local STATUS_KEY = "ntopng.cache.category_lists.status"
 
 local trace_level =  TRACE_INFO -- TRACE_NORMAL
 
@@ -37,96 +37,87 @@ local SIXH_DOWNLOAD_INTERVAL        = 21600
 local MAX_TOTAL_IP_RULES = 1000000
 -- Domain rules are the most expensive.
 -- On average they take ~7.5 KB/domain. 40k rules are loaded in about 7 seconds.
-local MAX_TOTAL_DOMAIN_RULES = 90000
+local MAX_TOTAL_DOMAIN_RULES = 200000
 -- JA3 rules use hash tables, so they are fast to load
 local MAX_TOTAL_JA3_RULES = 200000
 
 local is_nedge = ntop.isnEdge()
 
--- supported formats: ip, domain, hosts
+-- supported formats: ip, ip_csv, domain, hosts
 --
 -- Examples:
 --    [ip] 1.2.3.4
 --    [ip] 1.2.3.0/24
+--    [ip_csv] 0,216.245.221.83,0.0962959583990113 (Number,IP address,Rating)
 --    [domain] amalwaredomain.com
 --    [hosts] 127.0.0.1   amalwaredomain.com
 --    [hosts] 127.0.0.1   1.2.3.4
 --
-local BUILTIN_LISTS = {
-   ["ntop IP Malware Meltdown"] = {
-      url = "http://blacklists.ntop.org/blacklist-ip.txt",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "ip",
-      enabled = false,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["ntop Host Malware Meltdown"] = {
-      url = "http://blacklists.ntop.org/blacklist-hostnames.txt",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "domain",
-      enabled = false,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["Emerging Threats"] = {
-      url = "https://rules.emergingthreats.net/fwrules/emerging-Block-IPs.txt",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "ip",
-      enabled = true,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["Snort IP Block List"] = {
-      url = "https://snort.org/downloads/ip-block-list",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "ip",
-      enabled = false,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["Feodo Tracker Botnet C2 IP Blocklist"] = {
-      url = "https://feodotracker.abuse.ch/downloads/ipblocklist.txt",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "ip",
-      enabled = true,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["SSLBL Botnet C2 IP Blacklist"] = {
-      url = "https://sslbl.abuse.ch/blacklist/sslipblacklist.txt",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "ip",
-      enabled = true,
-      update_interval = SIXH_DOWNLOAD_INTERVAL,
-   }, ["Anti-WebMiner"] = {
-      url = "https://raw.githubusercontent.com/greatis/Anti-WebMiner/master/hosts",
-      category = CUSTOM_CATEGORY_MINING,
-      format = "hosts",
-      enabled = false,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["NoCoin Filter List"] = {
-      url = "https://raw.githubusercontent.com/hoshsadiq/adblock-nocoin-list/master/hosts.txt",
-      category = CUSTOM_CATEGORY_MINING,
-      format = "hosts",
-      enabled = true,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["Abuse.ch URLhaus"] = {
-      url = "https://urlhaus.abuse.ch/downloads/hostfile/",
-      category = CUSTOM_CATEGORY_MALWARE,
-      format = "hosts",
-      enabled = true,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["Disconnect.me Simple Ad List"] = {
-      url = "https://s3.amazonaws.com/lists.disconnect.me/simple_ad.txt",
-      category = CUSTOM_CATEGORY_ADVERTISEMENT,
-      format = "domain",
-      enabled = is_nedge,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["AdAway default blocklist"] = {
-      url = "https://adaway.org/hosts.txt",
-      category = CUSTOM_CATEGORY_ADVERTISEMENT,
-      format = "hosts",
-      enabled = is_nedge,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }, ["SSLBL JA3"] = {
-      url = "https://sslbl.abuse.ch/blacklist/ja3_fingerprints.csv",
-      format = "ja3_suricata_csv",
-      category = CUSTOM_CATEGORY_MALWARE,
-      enabled = false,
-      update_interval = DEFAULT_UPDATE_INTERVAL,
-   }
-}
+
+-- ##############################################
+
+local function parse_lists_from_dir(where)
+  local files = ntop.readdir(where)
+  local ret = {}
+  
+  for _,f in pairs(files) do
+    if(string.ends(f, ".list")) then
+      local path = where .. "/" .. f
+      local content = file_utils.read_file(path)
+      local j = json.decode(content)
+      
+      if(j == nil) then
+	 traceError(TRACE_WARNING, TRACE_CONSOLE, "Skipping invalid list "..path..": parse error")
+      else
+        -- Fix glitches
+        local skip = false
+
+        if(j.category == nil) then
+          traceError(TRACE_WARNING, TRACE_CONSOLE, "Skipping invalid list "..path ..": no category")
+          skip = true   
+
+        else 
+         local category = string.lower(tostring(j.category))
+         if(category == "mining")        then j.category = CUSTOM_CATEGORY_MINING
+         elseif(category == "malware")       then j.category = CUSTOM_CATEGORY_MALWARE
+         elseif(category == "advertisement") then j.category = CUSTOM_CATEGORY_ADVERTISEMENT
+         else
+            traceError(TRACE_WARNING, TRACE_CONSOLE, "Skipping invalid list "..path ..": invalid category ".. j.category)
+            skip = true
+         end
+        end
+
+        if(not(skip) and (j.name == nil)) then
+          traceError(TRACE_WARNING, TRACE_CONSOLE, "Skipping invalid list "..path ..": missing name")
+          skip = true
+        end
+        
+        if(not(skip)) then
+          ret[j.name] = j
+        end
+      end
+    end
+  end
+
+  return(ret)
+end
+
+-- ##############################################
+
+local cached_lists = nil
+
+local function get_lists()
+  if(cached_lists == nil) then
+    local lists_dir = dirs.installdir .. "/" .. "httpdocs/misc/lists"
+
+    local builtin = parse_lists_from_dir(lists_dir .. "/builtin")
+    local custom  = parse_lists_from_dir(lists_dir .. "/custom")
+
+    cached_lists = table.merge(builtin, custom)
+  end
+
+  return cached_lists
+end
 
 -- ##############################################
 
@@ -145,7 +136,7 @@ local function loadListsFromRedis()
       lists = json.decode(lists_metadata)
    end
 
-   lists = table.merge(BUILTIN_LISTS, lists)
+   lists = table.merge(get_lists(), lists)
 
    if((lists == nil) or (status == nil)) then
       return {}
@@ -153,7 +144,7 @@ local function loadListsFromRedis()
 
    for list_name, list in pairs(lists) do
       if status[list_name] then
-	 list.status = status[list_name]
+         list.status = status[list_name]
       end
    end
 
@@ -180,9 +171,10 @@ end
 -- @note see saveListsStatusToRedis for the list status
 local function saveListsMetadataToRedis(lists)
    local metadata = {}
-
+   local all_lists = get_lists()
+   
    for list_name, list in pairs(lists or {}) do
-      local default_prefs = BUILTIN_LISTS[list_name]
+      local default_prefs = all_lists[list_name]
       local meta = {}
       local has_custom_pref = false
 
@@ -208,12 +200,15 @@ function lists_utils.getCategoryLists()
    -- TODO add support for user defined urls
    local lists = {}
    local redis_lists = loadListsFromRedis()
-
+   local all_lists = get_lists()
+   local blacklists_stats = ntop.getBlacklistStats()
+   
    local default_status = {last_update=0, num_hosts=0, last_error=false, num_errors=0}
 
-   for key, default_values in pairs(BUILTIN_LISTS) do
+   for key, default_values in pairs(all_lists) do
       local list = table.merge(default_values, redis_lists[key] or {status = {}})
       list.status = table.merge(default_status, list.status)
+      list.status.num_hits = blacklists_stats[key] or 0
       lists[key] = list
       list.name = key
    end
@@ -261,6 +256,9 @@ local function initListCacheDir()
    ntop.mkdir(os_utils.fixPath(string.format("%s/category_lists", dirs.workingdir)))
 end
 
+-- ##############################################
+
+
 local function getListCacheFile(list_name, downloading)
    local f = string.format("%s/category_lists/%s.txt", dirs.workingdir, list_name)
 
@@ -300,6 +298,8 @@ local function getNextListUpdate(list)
    return next_update
 end
 
+-- ##############################################
+
 -- Returns true if the given list should be updated
 function shouldUpdate(list_name, list, now)
    local list_file
@@ -327,15 +327,15 @@ function shouldUpdate(list_name, list, now)
       tprint('---------------')
 
       tprint(((now >= next_update) or
-	       (not ntop.exists(list_file) and (list.status.num_errors < MAX_LIST_ERRORS)) or
-	       (ntop.getCache("ntopng.cache.category_lists.update." .. list_name) == "1")))
+               (not ntop.exists(list_file) and (list.status.num_errors < MAX_LIST_ERRORS)) or
+               (ntop.getCache("ntopng.cache.category_lists.update." .. list_name) == "1")))
       return(false)
    else
       -- note: num_errors is used to avoid retying downloading the same list again when
       -- the file does not exist
       return(((now >= next_update) or
-	       (not ntop.exists(list_file) and (list.status.num_errors < MAX_LIST_ERRORS)) or
-	       (ntop.getCache("ntopng.cache.category_lists.update." .. list_name) == "1")))
+               (not ntop.exists(list_file) and (list.status.num_errors < MAX_LIST_ERRORS)) or
+               (ntop.getCache("ntopng.cache.category_lists.update." .. list_name) == "1")))
    end
 end
 
@@ -358,78 +358,74 @@ local function checkListsUpdate(timeout)
       local list_file = getListCacheFile(list_name, false)
 
       if(shouldUpdate(list_name, list, now)) then
-	 local temp_fname = getListCacheFile(list_name, true)
-	 local msg = string.format("Updating list '%s' [%s]... ", list_name, list.url)
+         local temp_fname = getListCacheFile(list_name, true)
+         local msg = string.format("Updating list '%s' [%s]... ", list_name, list.url)
 
-	 traceError(trace_level, TRACE_INFO, string.format("Updating list '%s'... ", list_name))
+         traceError(trace_level, TRACE_INFO, string.format("Updating list '%s'... ", list_name))
 
-	 local started_at = os.time()
-	 local res = ntop.httpFetch(list.url, temp_fname, timeout)
+         local started_at = os.time()
+         local res = ntop.httpFetch(list.url, temp_fname, timeout)
 
-	 if(res and (res["RESPONSE_CODE"] == 200)) then
-	    -- download was successful, replace the original file
-	    os.rename(temp_fname, list_file)
-	    list.status.last_error = false
-	    list.status.num_errors = 0
-	    needs_reload = true
+         if(res and (res["RESPONSE_CODE"] == 200)) then
+            -- download was successful, replace the original file
+            os.rename(temp_fname, list_file)
+            list.status.last_error = false
+            list.status.num_errors = 0
+            needs_reload = true
 
-	    local alert = alert_consts.alert_types.alert_list_download_succeeded.new(
-	       list_name
-	    )
+            local alert = alert_consts.alert_types.alert_list_download_succeeded.new(list_name)
+            alert:set_score_notice()
+            alert:store(alerts_api.systemEntity(list_name))
 
-	    alert:set_score_notice()
+            msg = msg .. "OK"
+         else
+            -- failure
+            local respcode = 0
+            local last_error = i18n("delete_data.msg_err_unknown")
 
-	    alert:store(alerts_api.systemEntity(list_name))
+            if res and res["ERROR"] then
+               last_error = res["ERROR"]
+            elseif res and res["RESPONSE_CODE"] ~= nil then
+               respcode = ternary(res["RESPONSE_CODE"], res["RESPONSE_CODE"], "-")
 
-	    msg = msg .. "OK"
-	 else
-	    -- failure
-	    local respcode = 0
-	    local last_error = i18n("delete_data.msg_err_unknown")
+               if res["IS_PARTIAL"] then
+                  last_error = i18n("category_lists.connection_time_out", {duration=(os.time() - started_at)})
+               else
+                  last_error = i18n("category_lists.server_returned_error")
+               end
 
-	    if res and res["ERROR"] then
-	       last_error = res["ERROR"]
-	    elseif res and res["RESPONSE_CODE"] ~= nil then
-	       respcode = ternary(res["RESPONSE_CODE"], res["RESPONSE_CODE"], "-")
+               if(respcode > 0) then
+                  last_error = string.format("%s %s", last_error, i18n("category_lists.http_code", {err_code = respcode}))
+               end
+            end
 
-	       if res["IS_PARTIAL"] then
-		  last_error = i18n("category_lists.connection_time_out", {duration=(os.time() - started_at)})
-	       else
-		  last_error = i18n("category_lists.server_returned_error")
-	       end
+            list.status.last_error = last_error
+            list.status.num_errors = list.status.num_errors + 1
 
-	       if(respcode > 0) then
-		  last_error = string.format("%s %s", last_error, i18n("category_lists.http_code", {err_code = respcode}))
-	       end
-	    end
+            local alert = alert_consts.alert_types.alert_list_download_failed.new(
+               list_name,
+               last_error
+            )
 
-	    list.status.last_error = last_error
-	    list.status.num_errors = list.status.num_errors + 1
+            alert:set_score_error()
 
-	    local alert = alert_consts.alert_types.alert_list_download_failed.new(
-	       list_name,
-	       last_error
-	    )
+            alert:store(alerts_api.systemEntity(list_name))
 
-	    alert:set_score_error()
+            msg = msg .. "ERROR ["..last_error.."]"
+         end
 
-	    alert:store(alerts_api.systemEntity(list_name))
+         traceError(TRACE_NORMAL, TRACE_CONSOLE, msg)
 
-	    msg = msg .. "ERROR ["..last_error.."]"
-	 end
+         now = os.time()
+         -- set last_update even on failure to avoid blocking on the same list again
+         list.status.last_update = now
+         ntop.delCache("ntopng.cache.category_lists.update." .. list_name)
 
-	 traceError(TRACE_NORMAL, TRACE_CONSOLE, msg)
-
-	 now = os.time()
-	 -- set last_update even on failure to avoid blocking on the same list again
-	 list.status.last_update = now
-	 ntop.delCache("ntopng.cache.category_lists.update." .. list_name)
-
-	 if now-begin_time >= timeout then
-	    -- took too long, will resume on next housekeeping execution
-	    all_processed = false
-	    break
-	 end
+         if now-begin_time >= timeout then
+            -- took too long, will resume on next housekeeping execution
+            all_processed = false
+            break
+         end
       end
    end
 
@@ -439,20 +435,20 @@ local function checkListsUpdate(timeout)
    if(not all_processed) then
       -- Still in progress, do not mark as finished yet
       if(needs_reload) then
-	 -- cache this for the next invocation of checkListsUpdate as
-	 -- we are still in progress
-	 ntop.setCache("ntopng.cache.category_lists.needs_reload", "1")
+         -- cache this for the next invocation of checkListsUpdate as
+         -- we are still in progress
+         ntop.setCache("ntopng.cache.category_lists.needs_reload", "1")
       end
 
       return {
-	 in_progress = true
+         in_progress = true
       }
    else
       ntop.delCache("ntopng.cache.category_lists.needs_reload")
 
       return {
-	 in_progress = false,
-	 needs_reload = needs_reload,
+         in_progress = false,
+         needs_reload = needs_reload,
       }
    end
 end
@@ -467,11 +463,11 @@ local function loadWarning(msg)
       return
    end
 
-   traceError(TRACE_WARNING, TRACE_CONSOLE, msg)
+   traceError(TRACE_NORMAL, TRACE_CONSOLE, msg)
    cur_load_warnings = cur_load_warnings + 1
 end
 
---@return nil on parse error, "domain" if the loaded item is an host, "ip" otherwise
+--@return nil on parse error, "domain" if the loaded item is an host, "ip" or "ip_csv" otherwise
 local function loadListItem(host, category, user_custom_categories, list, num_line)
    category = tonumber(category)
 
@@ -485,35 +481,54 @@ local function loadListItem(host, category, user_custom_categories, list, num_li
 
       -- Checking for "whitelisted hosts"
       if user_custom_categories[category] ~= nil then
-	 local hosts_map = swapKeysValues(user_custom_categories[category])
-	 if hosts_map["!"..host] ~= nil then
-	    return nil
-	 end
+         local hosts_map = swapKeysValues(user_custom_categories[category])
+         if hosts_map["!"..host] ~= nil then
+            return nil
+         end
       end
 
       if isIPv4(host) or isIPv4Network(host) then
-	 -- IPv4 address
-	 if((not list) or (list.format ~= "domain")) then
-	   if((host == "0.0.0.0") or (host == "0.0.0.0/0") or (host == "255.255.255.255")) then
-	     loadWarning(string.format("Bad IPv4 address '%s' in list '%s'", host, list.name))
-	   else
-	     ntop.loadCustomCategoryIp(host, category)
-	     return "ip"
-	   end
-	 else
-	   loadWarning(string.format("Invalid IPv4 address '%s' in list '%s'", host, list.name))
-	 end
+         -- IPv4 address
+         if((not list) or (list.format ~= "domain")) then
+	    local h = split(host, ":") -- Handle hosts with ports
+
+	    host = h[1]
+            if((host == "0.0.0.0") or (host == "0.0.0.0/0") or (host == "255.255.255.255")) then
+               loadWarning(string.format("Bad IPv4 address '%s' in list '%s'", host, list.name))
+            else
+               if (list and list.name) then
+                  if not ntop.loadCustomCategoryIp(host, category, list.name) then
+                     loadWarning(string.format("Failure loading IP '%s' category '%s' in list '%s'", host, category, list.name))
+		  else
+		     if((category == CUSTOM_CATEGORY_MALWARE) and isLocal(host)) then
+			local alert = alert_consts.alert_types.alert_local_host_blacklisted.new(list.name, host)
+			
+			alert:set_score_error()
+			alert:store(alerts_api.systemEntity(list.name, host))
+
+			-- loadWarning(string.format("Found local IP '%s' in malware list '%s'", host, list.name))
+		     end
+                  end
+               end
+       
+               return "ip"
+            end
+         else
+            loadWarning(string.format("Invalid IPv4 address '%s' in list '%s'", host, list.name))
+         end
       elseif isIPv6(host) then
-	 -- IPv6 address
+         -- IPv6 address
          loadWarning(string.format("Unsupported IPv6 address '%s' found in list '%s'", host, list.name))
       else
-	 -- Domain
-	 if((not list) or (list.format ~= "ip")) then
-	    ntop.loadCustomCategoryHost(host, category)
-	    return "domain"
-	 else
-	   loadWarning(string.format("Invalid domain '%s' in list '%s'", host, list.name))
-	 end
+         -- Domain
+         if((not list) or (list.format ~= "ip")) then
+            if not ntop.loadCustomCategoryHost(host, category, list.name) then
+              loadWarning(string.format("Failure loading host '%s' category '%s' in list '%s'",  host, category, list.name))
+            end
+            return "domain"
+         else
+           loadWarning(string.format("Invalid domain '%s' in list '%s'", host, list.name))
+         end
       end
    end
 
@@ -530,7 +545,7 @@ local function parse_hosts_line(line)
       host = words[2]
 
       if((host == "localhost") or (host == "127.0.0.1") or (host == "::1")) then
-	 host = nil
+         host = nil
       end
    else
       -- invalid host
@@ -542,6 +557,26 @@ end
 
 -- ##############################################
 
+local function parse_ip_csv_line(line)
+   local words = string.split(line, ",")
+   local host = nil
+
+   if(words and (#words == 3)) then
+      host = words[2]
+
+      if((host == "localhost") or (host == "127.0.0.1") or (host == "::1")) then
+         host = nil
+      end
+   else
+      -- invalid host
+      host = nil
+   end
+   
+   return(host)
+end
+
+-- ##############################################
+
 local function handle_ja3_suricata_csv_line(line)
    local parts = string.split(line, ",")
 
@@ -549,8 +584,8 @@ local function handle_ja3_suricata_csv_line(line)
       local md5_hash = parts[1]
 
       if(string.len(md5_hash) == 32) then
-	 ntop.loadMaliciousJA3Hash(string.lower(md5_hash))
-	 return(true)
+         ntop.loadMaliciousJA3Hash(string.lower(md5_hash))
+         return(true)
       end
    end
 
@@ -578,58 +613,58 @@ local function loadFromListFile(list_name, list, user_custom_categories, stats)
       else -- Failure
          if list.status.num_hosts > 0 then
             -- Avoid generating warnings during first startup
-	    traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
+            traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
          end
 
          return(false)
       end
-
    else
-
       local f = io.open(list_fname, "r")
       local num_line = 0
 
       if f == nil then
          if list.status.num_hosts > 0 then
             -- Avoid generating warnings during first startup
-	    traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
+            traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("Could not find '%s'...", list_fname))
          end
 
          return(false)
       end
 
       for line in f:lines() do
-      	  num_line = num_line + 1
-         if ntop.isShutdown() then
-	    break
+                num_line = num_line + 1
+         if ntop.isShuttingDown() then
+            break
          end
          local trimmed = line:match("^%s*(.-)%s*$")
 
          if((string.len(trimmed) > 0) and not(string.starts(trimmed, "#"))) then
-	    local host = trimmed
+            local host = trimmed
 
-	    if list.format == "hosts" then
-	       host = parse_hosts_line(trimmed)
+            if(list.format == "hosts") then
+               host = parse_hosts_line(trimmed)
+	    elseif(list.format == "ip_csv") then
+	       host = parse_ip_csv_line(trimmed)
             end
 
-	    if host then
-	       local rv = loadListItem(host, list.category, user_custom_categories, list, num_line)
+            if host then
+               local rv = loadListItem(host, list.category, user_custom_categories, list, num_line)
 
-	       if(rv == "domain") then
-	          stats.num_hosts = stats.num_hosts + 1
-	          num_rules = num_rules + 1
-	       elseif(rv == "ip") then
-	          stats.num_ips = stats.num_ips + 1
-	          num_rules = num_rules + 1
-	       end
-	    end
+               if(rv == "domain") then
+                  stats.num_hosts = stats.num_hosts + 1
+                  num_rules = num_rules + 1
+               elseif(rv == "ip") then
+                  stats.num_ips = stats.num_ips + 1
+                  num_rules = num_rules + 1
+               end
+            end
 
-	    if((stats.num_ips >= MAX_TOTAL_IP_RULES) or
-	          (stats.num_hosts >= MAX_TOTAL_DOMAIN_RULES) or
-	          (stats.num_ja3 >= MAX_TOTAL_JA3_RULES)) then
-	       limit_exceeded = true
-	       break
-	    end
+            if((stats.num_ips >= MAX_TOTAL_IP_RULES) or
+                  (stats.num_hosts >= MAX_TOTAL_DOMAIN_RULES) or
+                  (stats.num_ja3 >= MAX_TOTAL_JA3_RULES)) then
+               limit_exceeded = true
+               break
+            end
          end
       end
 
@@ -640,8 +675,8 @@ local function loadFromListFile(list_name, list, user_custom_categories, stats)
 
    traceError(trace_level, TRACE_CONSOLE, string.format("\tRead '%d' rules", num_rules))
 
-   if((num_rules == 0) and (not limit_exceeded) and (not ntop.isShutdown())) then
-      traceError(TRACE_WARNING, TRACE_CONSOLE, string.format("List '%s' has 0 rules. Please report this to https://github.com/ntop/ntopng", list_name))
+   if((num_rules == 0) and (not limit_exceeded) and (not ntop.isShuttingDown())) then
+      traceError(TRACE_NORMAL, TRACE_CONSOLE, string.format("List '%s' has 0 rules. Please report this to https://github.com/ntop/ntopng", list_name))
    end
 
    return(limit_exceeded)
@@ -667,32 +702,32 @@ local function reloadListsNow()
    -- Load hosts from cached URL lists
    for list_name, list in pairsByKeys(lists) do
       if list.enabled then
-	 if((not limit_reached_error) and loadFromListFile(list_name, list, user_custom_categories, stats)) then
-	    -- A limit was exceeded
-	    if(stats.num_ips >= MAX_TOTAL_IP_RULES) then
-	       limit_reached_error = i18n("category_lists.too_many_ips_loaded", {limit = MAX_TOTAL_IP_RULES}) ..
-		  ". " .. i18n("category_lists.disable_some_list")
-	    elseif(stats.num_hosts >= MAX_TOTAL_DOMAIN_RULES) then
-	       limit_reached_error = i18n("category_lists.too_many_hosts_loaded", {limit = MAX_TOTAL_DOMAIN_RULES}) ..
-		  ". " .. i18n("category_lists.disable_some_list")
-	    elseif(stats.num_ja3 >= MAX_TOTAL_JA3_RULES) then
-	       limit_reached_error = i18n("category_lists.too_many_ja3_loaded", {limit = MAX_TOTAL_JA3_RULES}) ..
-		  ". " .. i18n("category_lists.disable_some_list")
-	    else
-	       -- should never happen
-	       limit_reached_error = "reloadListsNow: unknown error"
-	    end
+         if((not limit_reached_error) and loadFromListFile(list_name, list, user_custom_categories, stats)) then
+            -- A limit was exceeded
+            if(stats.num_ips >= MAX_TOTAL_IP_RULES) then
+               limit_reached_error = i18n("category_lists.too_many_ips_loaded", {limit = MAX_TOTAL_IP_RULES}) ..
+                  ". " .. i18n("category_lists.disable_some_list")
+            elseif(stats.num_hosts >= MAX_TOTAL_DOMAIN_RULES) then
+               limit_reached_error = i18n("category_lists.too_many_hosts_loaded", {limit = MAX_TOTAL_DOMAIN_RULES}) ..
+                  ". " .. i18n("category_lists.disable_some_list")
+            elseif(stats.num_ja3 >= MAX_TOTAL_JA3_RULES) then
+               limit_reached_error = i18n("category_lists.too_many_ja3_loaded", {limit = MAX_TOTAL_JA3_RULES}) ..
+                  ". " .. i18n("category_lists.disable_some_list")
+            else
+               -- should never happen
+               limit_reached_error = "reloadListsNow: unknown error"
+            end
 
-	    -- Continue to iterate to also set the error on the next lists
-	    traceError(TRACE_WARNING, TRACE_CONSOLE, limit_reached_error)
-	 end
+            -- Continue to iterate to also set the error on the next lists
+            traceError(TRACE_WARNING, TRACE_CONSOLE, limit_reached_error)
+         end
 
-	 if(limit_reached_error) then
-	    -- Set the invalid status to show it into the gui
-	    list.status.last_error = limit_reached_error
+         if(limit_reached_error) then
+            -- Set the invalid status to show it into the gui
+            list.status.last_error = limit_reached_error
 
-	    traceError(trace_level, TRACE_CONSOLE, limit_reached_error)
-	 end
+            traceError(trace_level, TRACE_CONSOLE, limit_reached_error)
+         end
       end
    end
 
@@ -702,10 +737,10 @@ local function reloadListsNow()
    -- Load user-customized categories
    for category_id, hosts in pairs(user_custom_categories) do
       for _, host in ipairs(hosts) do
-	 if ntop.isShutdown() then
-	    break
-	 end
-	 loadListItem(host, category_id, user_custom_categories, {} --[[ No list --]], 0 --[[ No line number --]])
+         if ntop.isShuttingDown() then
+            break
+         end
+         loadListItem(host, category_id, user_custom_categories, { name = "__gui__" } --[[ No list --]], 0 --[[ No line number --]])
       end
    end
 
@@ -716,8 +751,8 @@ local function reloadListsNow()
    stats.duration = (os.time() - stats.begin)
 
    traceError(TRACE_NORMAL, TRACE_CONSOLE,
-	      string.format("Category Lists (%u hosts, %u IPs, %u JA3) loaded in %d sec",
-			    stats.num_hosts, stats.num_ips, stats.num_ja3, stats.duration))
+              string.format("Category Lists (%u hosts, %u IPs, %u JA3) loaded in %d sec",
+                            stats.num_hosts, stats.num_ips, stats.num_ja3, stats.duration))
 
    -- Save the stats
    ntop.setCache("ntopng.cache.category_lists.load_stats", json.encode(stats))
@@ -753,8 +788,8 @@ function lists_utils.checkReloadLists()
       local rv = checkListsUpdate(60 --[[ timeout ]])
 
       if(not rv.in_progress) then
-	 ntop.delCache("ntopng.cache.download_lists_utils")
-	 reload_now = forced_reload or rv.needs_reload
+         ntop.delCache("ntopng.cache.download_lists_utils")
+         reload_now = forced_reload or rv.needs_reload
       end
    else
       reload_now = forced_reload
@@ -766,13 +801,13 @@ function lists_utils.checkReloadLists()
       -- print("[DEBUG] **** Reloading ****\n")
 
       if reloadListsNow() then
-	 -- print("[DEBUG]  Success !!!!\n")
-	 -- success
-	 ntop.delCache("ntopng.cache.reload_lists_utils")
+         -- print("[DEBUG]  Success !!!!\n")
+         -- success
+         ntop.delCache("ntopng.cache.reload_lists_utils")
       else
-	 -- print("[DEBUG]  ERROR !!!!\n")
-	 -- Remember to load the lists next time
-	 ntop.setCache("ntopng.cache.reload_lists_utils", "1")
+         -- print("[DEBUG]  ERROR !!!!\n")
+         -- Remember to load the lists next time
+         ntop.setCache("ntopng.cache.reload_lists_utils", "1")
       end
 
       -- print("[DEBUG] **** Reloading is over ****\n")
@@ -782,6 +817,17 @@ end
 -- ##############################################
 
 function lists_utils.startup()
+   local all_lists = get_lists()
+
+   -- tprint(all_lists)
+   
+   if ntop.isOffline() then
+      traceError(TRACE_NORMAL, TRACE_CONSOLE, "Category lists not loaded (offline)")
+      -- Reload the last list version as we're offline
+      reloadListsNow()
+      return
+   end
+
    traceError(TRACE_NORMAL, TRACE_CONSOLE, "Refreshing category lists...")
 
    lists_utils.downloadLists()
