@@ -1,12 +1,12 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
 local info = ntop.getInfo()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
-local plugins_utils = require("plugins_utils")
+local script_manager = require("script_manager")
 local template_utils = require("template_utils")
 local recording_utils = require("recording_utils")
 local alert_entities = require("alert_entities")
@@ -21,10 +21,24 @@ local active_section = nil
 local active_entry = nil
 local active_sub_entry = nil
 
+
+local zoneinfo = 'null'
+local temp_zoneinfo = ntop.getInfo().zoneinfo
+local newline = string.find(temp_zoneinfo, "\n")
+
+-- WARNING: in FreeBSD,	the zoneinfo ends with a newline character, so remove the newline and everything works fine
+if newline then
+  temp_zoneinfo = string.sub(temp_zoneinfo, 1, newline - 1)
+end
+
+if temp_zoneinfo ~= nil then
+  zoneinfo = "( /^[\\x00-\\xFF]*$/.test('" .. temp_zoneinfo .."') && moment.tz.zone('" .. temp_zoneinfo .."') != null ? '".. temp_zoneinfo .. "' : function() { console.warn('zoneinfo not valid:' + (/^[\\x00-\\xFF]*$/.test('" .. temp_zoneinfo .."') ? '" .. temp_zoneinfo .."' : 'Invalid string') ); return null; }())"
+end
+
 -- #################################
 
 page_utils.menu_sections = {
-   shortcuts     = {key = "shortcut", i18n_title = "shortcuts", icon = "fas fa-fast-forward"},
+   monitoring     = {key = "monitoring", i18n_title = "monitoring", icon = "fas fa-satellite-dish"},
    dashboard     = {key = "dashboard", i18n_title = "index_page.dashboard", icon = "fas fa-tachometer-alt"},
    alerts        = {key = "alerts", i18n_title = "details.alerts", icon = "fas fa-exclamation-triangle"},
    flows         = {key = "flows", i18n_title = "flows", icon = "fas fa-stream"},
@@ -35,6 +49,7 @@ page_utils.menu_sections = {
    system_stats  = {key = "system_stats", i18n_title = "system", icon = "fas fa-desktop"},
    admin         = {key = "admin", i18n_title = "settings", icon = "fas fa-cog"},
    alert_exclusions = {key = "alert_exclusions", i18n_title = "edit_check.exclusion_list", icon = "fas fa-bell-slash"},
+   device_exclusions = {key = "device_exclusions", i18n_title = "edit_check.device_exclusions", icon = "fas fa-bell-slash"},
    dev           = {key = "dev", i18n_title = "developer", icon = "fas fa-code"},
    about         = {key = "about", i18n_title = "help", icon = "fas fa-life-ring"},
    health        = {key = "health", i18n_title = "health", icon = "fas fa-laptop-medical"},
@@ -52,23 +67,27 @@ page_utils.menu_sections = {
 -- help_link: this variable contains the contextual link to the documentation
 
 page_utils.menu_entries = {
-   -- Shortcuts
-   active_monitoring = {key = "active_monitor", i18n_title = "active_monitoring_stats.active_monitoring"},
-
+   -- Monitoring
+    active_monitoring     = {key = "active_monitoring", i18n_title = "active_monitoring_stats.active_monitoring", section = "monitoring" },
+    vulnerability_scan    = {key = "vulnerability_scan", i18n_title = "scan_hosts", section = "monitoring" },
    -- Dashboard
     traffic_dashboard     = {key = "traffic_dashboard", i18n_title = "dashboard.traffic_dashboard", section = "dashboard", help_link = "https://www.ntop.org/guides/ntopng/web_gui/dashboard.html"},
     network_discovery     = {key = "network_discovery", i18n_title = "discover.network_discovery",  section = "dashboard"},
     traffic_report        = {key = "traffic_report",    i18n_title = "report.traffic_report",    section = "dashboard", help_link = "https://www.ntop.org/guides/ntopng/web_gui/report.html"},
-    db_explorer           = {key = "db_explorer", i18n_title = "db_explorer.historical_data_explorer", section = "dashboard", help_link = "https://www.ntop.org/guides/ntopng/clickhouse.html?#historical-flows-explorer"},
+    traffic_analysis      = {key = "traffic_analysis", i18n_title = "db_explorer.traffic_analysis", section = "dashboard", help_link = "https://www.ntop.org/guides/ntopng/clickhouse.html?#historical-flows-explorer"},
 
     -- Alerts
     detected_alerts       = {key = "detected_alerts", i18n_title = "show_alerts.detected_alerts", section = "alerts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/alerts.html"},
     alerts_dashboard      = {key = "alerts_dashboard", i18n_title = "alerts_dashboard.alerts_dashboard", section = "alerts"},
     flow_alerts_explorer  = {key = "flow_alerts_explorer", i18n_title = "flow_alerts_explorer.label", section = "alerts"},
-
+    alerts_list           = {key = "alerts_list", i18n_title = "details.alerts_list", section = "alerts"},
+    alerts_analysis       = {key = "alerts_analysis", i18n_title = "details.alerts_analysis", section = "alerts"},
+    
     -- Flows
     flows                 = {key = "flows", i18n_title = "flows", section = "flows", help_link = "https://www.ntop.org/guides/ntopng/web_gui/flows.html"},
+    active_flows          = {key = "active_flows", i18n_title = "active_flows", section = "flows" },
     flow_details          = {key = "flow_details", i18n_title = "flow_details.flow_details", section = "flows"},
+    db_explorer           = {key = "db_explorer", i18n_title = "db_explorer.historical_data_explorer", section = "flows", help_link = "https://www.ntop.org/guides/ntopng/clickhouse.html?#historical-flows-explorer"},
 
     -- Hosts
     hosts                 = {key = "hosts", i18n_title = "hosts", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html#"},
@@ -79,14 +98,17 @@ page_utils.menu_entries = {
     autonomous_systems    = {key = "autonomous_systems", i18n_title = "as_stats.autonomous_systems", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#autonomous-systems"},
     countries             = {key = "countries", i18n_title = "countries", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#countries"},
     operating_systems     = {key = "operating_systems", i18n_title = "operating_systems", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#operating-systems"},
-    http_servers          = {key = "http_servers", i18n_title = "http_servers_stats.http_servers", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#http-servers-local"},
+    http_servers          = {key = "http_servers", i18n_title = "http_servers_stats.local_http_servers", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#http-servers-local"},
+    server_ports_analysis = {key = "server_ports_analysis", i18n_title = "server_ports_analysis", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#http-servers-local"},
     top_hosts             = {key = "top_hosts", i18n_title = "processes_stats.top_hosts", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#top-hosts-local"},
     hosts_treemap         = {key = "hosts_treemap", i18n_title = "tree_map.hosts_treemap", section = "hosts"},
     containers            = {key = "containers", i18n_title = "containers_stats.containers", section = "hosts"},
     pods                  = {key = "pods", i18n_title = "containers_stats.pods", section = "hosts"},
+    host_rules            = {key = "host_rules", i18n_title = "if_stats_config.traffic_rules", section = "hosts", help_link = "https://www.ntop.org/guides/ntopng/advanced_features/host_rules.html"},
 
     -- Interface
-    interface             = {key = "interface", i18n_title = "interface_ifname", section = "if_stats"},
+    interface             = {key = "interface", i18n_title = "interface_details", section = "if_stats"},
+    ports_analysis        = {key = "ports_analysis", i18n_title = "ports_analysis.server_ports", section = "if_stats"},
 
     -- Pollers
     snmp                  = {key = "snmp", i18n_title = "prefs.snmp", section = "pollers"},
@@ -96,7 +118,7 @@ page_utils.menu_entries = {
     system_status         = {key = "system_status", i18n_title = "system_status", section = "health"},
     interfaces_status     = {key = "interfaces_status", i18n_title = "system_interfaces_status", section = "health"},
     alerts_status         = {key = "alerts_status", i18n_title = "system_alerts_status", section = "health"},
-    influxdb_status       = {key = "influxdb", i18n_title = "InfluxDB", section = "health"},
+    influxdb_status       = {key = "influxdb_status", i18n_title = "InfluxDB", section = "health"},
     redis_status          = {key = "redis_monitor", i18n_title = "Redis", section = "health"},
     clickhouse_status     = {key = "clickhouse", i18n_title = "ClickHouse", section = "health"},
 
@@ -107,8 +129,7 @@ page_utils.menu_entries = {
     observation_points    = {key = "observation_points", i18n_title = "flow_devices.observation_points", section = "collection", help_link = "https://www.ntop.org"},
 
    -- Map
-   service_map            = {key = "service_map", i18n_title="service", section = "maps"},
-   periodicity_map        = {key = "periodicity_map", i18n_title="periodicity", section = "maps"},
+   analysis_map           = {key = "analysis_map", i18n_title="analysis", section = "maps"},
    geo_map                = {key = "geo_map", i18n_title = "geo_map.geo_map", section = "maps", help_link = "https://www.ntop.org/guides/ntopng/web_gui/hosts.html?#geo-map"},
    hosts_map              = {key = "hosts_map", i18n_title = "flows_page.hosts", section = "maps"},
    
@@ -125,6 +146,8 @@ page_utils.menu_entries = {
    scripts_config_flows   = {key = "scripts_config", subkey="flows", i18n_title = alert_entities.flow.i18n_label, section = "admin", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
    scripts_config_system  = {key = "scripts_config", subkey="system", i18n_title = alert_entities.system.i18n_label, section = "admin", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
    scripts_config_syslog  = {key = "scripts_config", subkey="syslog", i18n_title = "syslog.syslog", section = "admin", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
+   scripts_config_active_monitoring  = {key = "scripts_config", subkey="active_monitoring", i18n_title = "active_monitoring", section = "admin", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
+   device_exclusions      = {key = "device_exclusions", i18n_title = "edit_check.device_exclusion", section = "device_exclusions"},
    alert_exclusions       = {key = "alert_exclusions", i18n_title = "edit_check.exclusion_list", section = "alert_exclusions", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
    alert_exclusions_hosts = {key = "alert_exclusions", subkey="hosts", i18n_title = alert_entities.host.i18n_label, section = "alert_exclusions", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
    alert_exclusions_flows = {key = "alert_exclusions", subkey="flows", i18n_title = alert_entities.flow.i18n_label, section = "alert_exclusions", help_link = "https://www.ntop.org/guides/ntopng/web_gui/checks.html"},
@@ -140,8 +163,8 @@ page_utils.menu_entries = {
    manage_configurations  = {key = "manage_configurations", i18n_title = "manage_configurations.manage_configurations", section = "admin"},
 
    -- Notifications
-   endpoint_notifications = {key = "endpoint_notifications", i18n_title = "endpoint_notifications.endpoint_list", section="notifications", help_link = "https://www.ntop.org/guides/ntopng/plugins/alert_endpoints.html"},
-   endpoint_recipients    = {key = "endpoint_recipients", i18n_title = "endpoint_notifications.enpoint_recipients_list", section="notifications", help_link = "https://www.ntop.org/guides/ntopng/plugins/alert_endpoints.html"},
+   endpoint_notifications = {key = "endpoint_notifications", i18n_title = "endpoint_notifications.endpoint_list", section="notifications", help_link = "https://www.ntop.org/guides/ntopng/scripts/alert_endpoints.html"},
+   endpoint_recipients    = {key = "endpoint_recipients", i18n_title = "endpoint_notifications.enpoint_recipients_list", section="notifications", help_link = "https://www.ntop.org/guides/ntopng/scripts/alert_endpoints.html"},
 
    -- Tools
    export_data            = {key = "export_data", i18n_title = "manage_data.export", section = "tools"},
@@ -154,16 +177,15 @@ page_utils.menu_entries = {
    live_capture           = {key = "live_capture", i18n_title = "live_capture.active_live_captures", section = "home"},
 
    -- Developer
-   directories            = {key = "directories", i18n_title = "about.directories", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/plugins/distributing_plugins.html"},
-   plugins                = {key = "plugins", i18n_title = "plugins", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/basic_concepts/plugins.html"},
-   checks_dev             = {key = "checks_dev", i18n_title = "about.checks", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/plugins/checks.html"},
-   plugin_browser         = {key = "plugin_browser", i18n_title = "plugin_browser", section = "dev"},
-   alert_definitions      = {key = "alert_definitions", i18n_title = "about.alert_defines", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/plugins/alert_definitions.html"},
+   directories            = {key = "directories", i18n_title = "about.directories", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/scripts/distributing_scripts.html"},
+   checks_dev             = {key = "checks_dev", i18n_title = "about.checks", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/scripts/checks.html"},
+   analyze_pcap           = {key = "analyze_pcap", i18n_title = "about.analyze_pcap", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/advanced_features/index.html"},
+   alert_definitions      = {key = "alert_definitions", i18n_title = "about.alert_defines", section = "dev", help_link = "https://www.ntop.org/guides/ntopng/scripts/alert_definitions.html"},
    api                    = {key = "api", i18n_title = "lua_c_api", section = "dev"},
+   rest_api               = {key = "rest_api", i18n_title = "swagger_api", section = "dev"},
    
    -- Help
    about                  = {key = "about", i18n_title = "about.about", section = "about", help_link = "https://www.ntop.org/guides/ntopng/web_gui/help_menu.html?#about"},
-   telemetry              = {key = "telemetry", i18n_title = "telemetry", section = "about"},
    blog                   = {key = "blog", i18n_title = "about.ntop_blog", section = "about"},
    telegram               = {key = "telegram", i18n_title = "about.telegram", section = "about"},
    report_issue           = {key = "report_issue", i18n_title = "about.report_issue", section = "about"},
@@ -182,11 +204,13 @@ page_utils.menu_entries = {
    dhcp_static_leases     = {key = "dhcp_static_leases", i18n_title = "nedge.static_dhcp_leases", section = "system_stats"},
    dhcp_active_leases     = {key = "dhcp_active_leases", i18n_title = "nedge.active_dhcp_leases", section = "system_stats"},
    port_forwarding        = {key = "port_forwarding", i18n_title = "nedge.port_forwarding", section = "system_stats"},
+   rules_config           = {key = "rules_config", i18n_title = "nedge.rules_config_title", section = "system_stats"},
+   forwarders_config       = {key = "forwarders_config", i18n_title = "nedge.repeaters_config_title", section = "system_stats"}
 }
 
--- Extend the menu entries with the plugins
-local menu, entries_data = plugins_utils.getMenuEntries()
-page_utils.plugins_menu = menu or {}
+-- Extend the menu entries with the scripts
+local menu, entries_data = script_manager.getMenuEntries()
+page_utils.scripts_menu = menu or {}
 
 if entries_data then
     for k, v in pairs(entries_data) do
@@ -196,14 +220,18 @@ end
 
 -- #################################
 
--- NOTE: this function must be called after page_utils.set_active_menu_entry
-function page_utils.print_page_title(title)
+-- NOTE: this function must be called after page_utils.print_header_and_set_active_menu_entry
+function page_utils.print_page_title(title, mini_title)
    local help_link = page_utils.menu_entries[active_entry].help_link or ""
    print("<header class='mb-3 d-flex align-items-center'>")
    print("<h2 class='d-inline-block'>".. title .."</h2>")
    if (not isEmptyString(help_link)) then
       print("<a data-bs-toggle='tooltip' title='".. i18n("open_documentation") .."' target='_newtab' href='".. help_link .."' class='text-muted ms-auto'><i class='fas fa-question-circle'></i></a>")
    end
+   
+   if mini_title and not isEmptyString(mini_title) then
+    print("<small style='margin-left: auto;'>" .. mini_title .. "</small>")
+   end
    print("</header>")
 
 end
@@ -212,7 +240,7 @@ end
 
 -- NOTE: this function is called by the web pages in order to
 -- set the active entry and section and highlight it into the menu
-function page_utils.set_active_menu_entry(entry, i18n_params, alt_title)
+function page_utils.print_header_and_set_active_menu_entry(entry, i18n_params, alt_title)
    entry = entry or page_utils.menu_entries.traffic_dashboard
 
    active_section = entry.section
@@ -280,7 +308,7 @@ end
 
 -- #################################
 
-function page_utils.print_header(title)
+function page_utils.print_header(title, addLoginJS)
   local http_prefix = ntop.getHttpPrefix()
   local static_file_epoch = ntop.getStaticFileEpoch()..""
 
@@ -288,6 +316,11 @@ function page_utils.print_header(title)
   local page_title = i18n("welcome_to", { product=info.product })
   local favicon_path = nil
 
+  local admin_lang = ntop.getPref("ntopng.user.admin.language")
+  local language = ternary(isEmptyString(admin_lang), "en", admin_lang)
+  local locale_path = dirs.installdir.."/scripts/locales/"..language..".lua"
+  local locale_when = ntop.fileLastChange(locale_path)
+
   if title ~= nil then
     page_title = info.product .. " - " .. title
   end
@@ -300,24 +333,16 @@ function page_utils.print_header(title)
     <meta name="viewport" content="width=device-width, initial-scale=1">
     <meta http-equiv="X-UA-Compatible" content="IE=edge">
 
-    <link href="]] print(http_prefix) print[[/bootstrap/css/bootstrap.min.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/minified/ntopng.min.css?]] print(static_file_epoch) print[[" rel="stylesheet"> ]]
-
+    <link href="]] print(http_prefix) print[[/dist/third-party.css?]] print(static_file_epoch) print[[" rel="stylesheet">
+    <link href="]] print(http_prefix) print[[/dist/ntopng.css?]] print(static_file_epoch) print[[" rel="stylesheet">
+    <link href="]] print(http_prefix) print[[/dist/white-mode.css?]] print(static_file_epoch) print[[" rel="stylesheet">]]
+    
     if (dark_mode) then
-      print[[<link href="]] print(http_prefix) print[[/css/minified/dark-mode.min.css?]] print(static_file_epoch) print[[" rel="stylesheet">]]
+      print[[<link href="]] print(http_prefix) print[[/dist/dark-mode.css?]] print(static_file_epoch) print[[" rel="stylesheet">]]
     end
 
-    print[[
-    <link href="]] print(http_prefix) print[[/fontawesome-free-5.11.2-web/css/all.min.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/minified/tempusdominus.min.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/minified/heatmap.min.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/minified/rickshaw.min.css" type="text/css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/dc.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/selectpicker/css/bootstrap-select.min.css" rel="stylesheet">
-    ]]
-
    if (ntop.isPro() or ntop.isnEdge()) and ntop.exists(dirs.installdir .. "/httpdocs/img/custom_favicon.ico") then
-      favicon_path = ntop.getHttpPrefix().."/img/custom_favicon.ico"
+      favicon_path = http_prefix.."/img/custom_favicon.ico"
    end
    if (favicon_path ~= nil) then
     print[[<link href="]] print(favicon_path) print[[" rel="icon">]]
@@ -326,28 +351,36 @@ function page_utils.print_header(title)
    end
 
    print[[
-   <style>
+    <style>
       .flag {
-         width: 16px;
-         height: 11px;
-         margin-top: -5px;
-         background:url(]] print(http_prefix) print[[/img/flags.png) no-repeat
+        width: 16px;
+        height: 11px;
+        margin-top: -5px;
+        background:url(]] print(http_prefix) print[[/dist/images/flags.png) no-repeat
       }
-   </style>
-    <link href="]] print(http_prefix) print[[/css/flags.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/pie-chart.css" rel="stylesheet">
-    <!-- http://kamisama.github.io/cal-heatmap/v2/ -->
-    <link href="]] print(http_prefix) print[[/css/nv.d3.css" rel="stylesheet">
-    <link href="]] print(http_prefix) print[[/css/custom_theme.css?]] print(static_file_epoch) print[[" rel="stylesheet">
-    <!--[if lt IE 9]>
-      <script src="]] print(http_prefix) print[[/js/html5shiv.js"></script>
-    <![endif]-->
-    <script type="text/javascript" src="]] print(http_prefix) print[[/js/jquery_bootstrap.min.js?]] print(static_file_epoch) print[["></script>
-    <script type="text/javascript" src="]] print(http_prefix) print[[/js/popper/popper.min.js" crossorigin="anonymous"></script>
-    <script type="text/javascript" src="]] print(http_prefix) print[[/js/deps.min.js?]] print(static_file_epoch) print[["></script>
-    <script type="text/javascript" src="]] print(http_prefix) print[[/js/ntop.min.js?]] print(static_file_epoch) print[["></script>
-    <script type="text/javascript" src="]] print(http_prefix) print[[/js/tempusdominus.min.js?]] print(static_file_epoch) print[["></script>
-  </head>]]
+    </style>
+
+    ]]
+          
+    if addLoginJS then
+      print[[<script type="application/javascript" src="]] print(http_prefix) print[[/dist/login.js?]] print(static_file_epoch) print[["></script>]]
+    end
+
+    if(ntop.isPro() and ntop.exists(dirs.installdir.."/css/custom-theme.css")) then
+      print [[
+        <link href="]] print(http_prefix) print[[/css/custom-theme.css?]] print(static_file_epoch) print[[" rel="stylesheet">
+      ]]
+    end
+    
+    print[[
+    <script type="application/javascript" src="]] print(http_prefix) print("/lua/locale.lua?"..locale_when .. "&user_language=" ..language); print[["> </script>
+    <script type="application/javascript" src="]] print(http_prefix) print[[/dist/third-party.js?]] print(static_file_epoch) print[["></script>
+    <script type="application/javascript" src="]] print(http_prefix) print[[/dist/ntopng.js?]] print(static_file_epoch) print[["></script>
+    <script>
+    const ntop_zoneinfo = ]] print(zoneinfo) print[[;
+    const http_prefix = "]] print(ntop.getHttpPrefix()) print [[";
+    </script>
+    </head>]]
   print([[
      <body class="body ]].. (dark_mode and "dark" or "") ..[[">
   ]])
@@ -355,14 +388,20 @@ end
 
 -- #################################
 
-function page_utils.print_header_minimal(title)
+function page_utils.print_header_minimal(title, addLoginJS)
   local http_prefix = ntop.getHttpPrefix()
 
   local page_title = i18n("welcome_to", { product=info.product })
   if title ~= nil then
     page_title = info.product .. " - " .. title
   end
-
+  
+  local admin_lang = ntop.getPref("ntopng.user.admin.language")
+  local language = ternary(isEmptyString(admin_lang), "en", admin_lang)
+  local locale_path = dirs.installdir.."/scripts/locales/"..language..".lua"
+  local locale_when = ntop.fileLastChange(locale_path)
+  local static_file_epoch = ntop.getStaticFileEpoch()..""
+  
   print [[
     <!DOCTYPE html>
     <html>
@@ -370,15 +409,13 @@ function page_utils.print_header_minimal(title)
         <title>]] print(page_title) print[[</title>
         <meta name="viewport" content="width=device-width, initial-scale=1">
         <meta http-equiv="X-UA-Compatible" content="IE=edge">
-        <link href="]] print(http_prefix) print[[/bootstrap/css/bootstrap.min.css" rel="stylesheet">
-        <link href="]] print(http_prefix) print[[/fontawesome-free-5.11.2-web/css/fontawesome.css" rel="stylesheet">
-        <link href="]] print(http_prefix) print[[/fontawesome-free-5.11.2-web/css/brands.css" rel="stylesheet">
-        <link href="]] print(http_prefix) print[[/fontawesome-free-5.11.2-web/css/solid.css" rel="stylesheet">
 
+        <link href="]] print(http_prefix) print[[/dist/third-party.css?]] print(static_file_epoch) print[[" rel="stylesheet">
+        <link href="]] print(http_prefix) print[[/dist/ntopng.css?]] print(static_file_epoch) print[[" rel="stylesheet">
           ]]
 
          if (ntop.isPro() or ntop.isnEdge()) and ntop.exists(dirs.installdir .. "/httpdocs/img/custom_favicon.ico") then
-            favicon_path = ntop.getHttpPrefix().."/img/custom_favicon.ico"
+            favicon_path = http_prefix.."/img/custom_favicon.ico"
          end
          if (favicon_path ~= nil) then
           print[[<link href="]] print(favicon_path) print[[" rel="icon">]]
@@ -387,19 +424,18 @@ function page_utils.print_header_minimal(title)
          end
 
          print[[
-
-        <script type="text/javascript" src="]] print(http_prefix) print[[/js/jquery_bootstrap.min.js?]] print(static_file_epoch) print[["></script>
-        <link type="text/css" rel="stylesheet" href="]] print(http_prefix) print[[/css/rickshaw.css">
-        <script src="]] print(http_prefix) print[[/js/validator.js"></script>
-        <style>
-        .flag {
-          width: 16px;
-          height: 11px;
-          margin-top: -5px;
-          background:url(]] print(http_prefix) print[[/img/flags.png) no-repeat
-        }
-        </style>
-        <link href="]] print(http_prefix) print[[/css/ntopng.css" rel="stylesheet">
+          <style>
+          .flag {
+            width: 16px;
+            height: 11px;
+            margin-top: -5px;
+            background:url(]] print(http_prefix) print[[/dist/images/flags.png) no-repeat
+          }
+          </style>
+          <script type="application/javascript" src="]] print(http_prefix) print("/lua/locale.lua?"..locale_when .. "&user_language=" ..language); print[["> </script>
+          <script type="application/javascript" src="]] print(http_prefix) print[[/dist/third-party.js?]] print(static_file_epoch) print[["></script>
+          <script type="application/javascript" src="]] print(http_prefix) print[[/dist/ntopng.js?]] print(static_file_epoch) print[["></script>
+          <script> const ntop_zoneinfo = ]] print(zoneinfo) print[[;</script>          
       </head>
       <body>
 
@@ -408,7 +444,44 @@ end
 
 -- #################################
 
-function page_utils.print_navbar(title, base_url, items_table, label_url, back_url)
+function page_utils.get_navbar_context(title, base_url, items_table, label_url, back_url)
+   local help_link = page_utils.menu_entries[active_entry].help_link or nil
+   local icon = page_utils.menu_sections[active_section].icon or ""
+
+   local navbar = {
+   	 main_icon   = icon,
+         main_title  = title,
+         base_url    = base_url,
+         items_table = items_table,
+         label_url   = label_url,
+         back_url    = back_url,
+         help_link   = help_link
+   }
+   return navbar
+end
+
+-- #################################
+
+function page_utils.get_new_navbar_context(title, base_url, items_table, label_url, back_url)
+   local help_link = page_utils.menu_entries[active_entry].help_link or nil
+   local icon = page_utils.menu_sections[active_section].icon or ""
+
+   local navbar = {
+    main_title = { 
+      icon = icon,
+      label  = title,
+    },
+    base_url    = base_url,
+    items_table = items_table,
+    label_url   = label_url,
+    back_url    = back_url,
+    help_link   = help_link
+   }
+   return navbar
+end
+-- #################################
+
+function page_utils.print_navbar(title, base_url, items_table, label_url, back_url, end_items)
    local help_link = page_utils.menu_entries[active_entry].help_link or nil
    local icon = page_utils.menu_sections[active_section].icon or ""
 
@@ -419,7 +492,8 @@ function page_utils.print_navbar(title, base_url, items_table, label_url, back_u
          items_table = items_table,
          label_url   = label_url,
          back_url    = back_url,
-         help_link   = help_link
+         help_link   = help_link,
+         end_items   = end_items
       }
    }
    print(template_utils.gen("pages/components/page-navbar.template", context))
@@ -714,7 +788,7 @@ function page_utils.get_interface_list()
       ifdescr[_ifstats.id] = _ifstats.description
       --io.write("["..k.."/"..v.."][".._ifstats.id.."] "..ifnames[_ifstats.id].."=".._ifstats.id.."\n")
       if(_ifstats.isView == true) then views[k] = true end
-      if(_ifstats.isDynamic == true) then dynamic[k] = true end
+      if(interface.isSubInterface()) then dynamic[k] = true end
       if(recording_utils.isEnabled(_ifstats.id)) then recording[k] = true end
       if(interface.isPacketInterface()) then packetinterfaces[k] = true end
       if(_ifstats.stats_since_reset.drops * 100 > _ifstats.stats_since_reset.packets) then
@@ -827,7 +901,7 @@ function page_utils.generate_info_stats()
    else
       return ([[
          <a href=']].. ntop.getHttpPrefix() ..[[/lua/if_stats.lua'>
-            <span style='display: none;' class="network-load-chart-total">0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0</span>
+            <span style='display: none;' class="network-load-chart-total">0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0</span>
             <span class="text-end chart-total-text"></span>
          </a>
       ]])
