@@ -1,27 +1,41 @@
 --
--- (C) 2014-18 - ntop.org
+-- (C) 2014-22 - ntop.org
 --
 
+local clock_start = os.clock()
+
 local dirs = ntop.getDirs()
 require "lua_utils"
---require "prefs_utils"
 local json = require("dkjson")
 local os_utils = require("os_utils")
+local scripts_triggers = require "scripts_triggers"
 
-prefs = ntop.getPrefs()
+local prefs = ntop.getPrefs()
 
 local extraction_queue_key = "ntopng.traffic_recording.extraction_queue"
 local extraction_stop_queue_key = "ntopng.traffic_recording.extraction_stop_queue"
 local extraction_seqnum_key = "ntopng.traffic_recording.extraction_seqnum"
 local extraction_jobs_key = "ntopng.traffic_recording.extraction_jobs"
-local IS_AVAILABLE_KEY = "ntopng.cache.traffic_recording_available"
+local is_available_key = "ntopng.cache.traffic_recording_available"
+local provider_key = "ntopng.prefs.traffic_recording.ifid_%d.provider"
+local external_providers_reminder_dismissed_key = "ntopng.prefs.traffic_recording.ifid_%d.reminder_dismissed"
+local is_running_job_pending_key = "ntopng.cache.traffic_recording_job_pending"
 
 local recording_utils = {}
 
-recording_utils.default_disk_space = 10*1024
+recording_utils.default_disk_space = 10 * 1024
 
-local function n2diskctl(...)
-  return os_utils.ntopctlCmd("n2disk", ...)
+-- #################################
+
+local function extraction_checks_to_i18n(err)
+   local map = {
+      ERR_MISSING_TIMELINE = "traffic_recording.msg_err_missing_timeline",
+      ERR_TIMELINE_PATH_NOT_EXISTING = "traffic_recording.msg_err_timeline_path_not_existing",
+      ERR_UNABLE_TO_ACCESS_TIMELINE = "traffic_recording.msg_err_unable_to_access_timeline",
+      OK = "traffic_recording.msg_ok",
+   }
+
+   return map[err] or 'traffic_recording.msg_err_unknown'
 end
 
 -- #################################
@@ -73,25 +87,62 @@ function recording_utils.isSupportedInterface(ifid)
   return false
 end
 
+--! @brief Check if a reminder that warns the user about available external traffic rec. providers has to be shown
+--! @return true if the reminder has to be shown, false otherwise
+function recording_utils.isExternalProvidersReminderDismissed(ifid)
+   local k = string.format(external_providers_reminder_dismissed_key, ifid)
+   local cur_pref = ntop.getPref(k)
+
+   if cur_pref == "true" then
+      return true -- reminder has been explicitly dismissed
+   end
+
+   if recording_utils.getCurrentTrafficRecordingProvider(ifid) ~= "ntopng" then
+      return true -- an external traffic recording provider has already been selected
+   end
+
+   local providers = recording_utils.getAvailableTrafficRecordingProviders()
+   if #providers == 1 then
+      return true -- there's only one provider
+   end
+
+   return false
+end
+
+--! @brief Dismiss the reminder for external traffic recording providers
+--! @return nil
+function recording_utils.dismissExternalProvidersReminder(ifid)
+   local k = string.format(external_providers_reminder_dismissed_key, ifid)
+   ntop.setPref(k, "true")
+end
+
 -- only called during boot
 function recording_utils.checkAvailable()
   local is_available = false
 
   if(not ntop.isWindows())
     and (not ntop.isnEdge())
-    and os_utils.hasService("n2disk", "dummy") then
+    and os_utils.hasService("n2disk-ntopng", "dummy") then
     is_available = true
   end
 
-  ntop.setCache(IS_AVAILABLE_KEY, ternary(is_available, "1", "0"))
+  ntop.setCache(is_available_key, ternary(is_available, "1", "0"))
+
+  -- forcing a setJobAsCompleted after startup to handle interrupted jobs
+  ntop.setCache(is_running_job_pending_key, "1")
 end
 
---! @brief Check if traffic recording is available and allowed for the current user on an interface
---! @return true if recording is available, false otherwise
 function recording_utils.isAvailable()
-  if isAdministrator() and (ntop.getCache(IS_AVAILABLE_KEY) == "1") then
+   return scripts_triggers.isRecordingAvailable()
+end
+
+--! @brief Check if traffic recording and extraction is allowed for the current user on an interface
+--! @return true if extraction is available, false otherwise
+function recording_utils.isExtractionAvailable()
+  if ntop.isPcapDownloadAllowed() and (ntop.getCache(is_available_key) == "1") then
     return true
   end
+
   return false
 end
 
@@ -129,8 +180,7 @@ end
 --! @param key The license key
 --! @return true if the license is installed, false in case it is not possible
 function recording_utils.setLicense(key)
-  n2diskctl("set-license", key)
-  return true
+   return os_utils.ntopctlCmd("n2disk-ntopng", "set-license", key)
 end
 
 local function setLicenseFromRedis()
@@ -153,13 +203,30 @@ local function isZCInterface(ifname)
   return false
 end
 
+local function isRecordingEnabledInCache(ifid)
+  local enabled = ntop.getCache('ntopng.prefs.ifid_'..ifid..'.traffic_recording.enabled')
+  if not isEmptyString(enabled) and 
+     (enabled == "true" or enabled == "1") then
+     return true
+  end
+  return false
+end
+
+local function isSmartRecordingEnabledInCache(ifid)
+  local enabled = ntop.getCache('ntopng.prefs.ifid_'..ifid..'.smart_traffic_recording.enabled')
+  if not isEmptyString(enabled) and 
+     (enabled == "true" or enabled == "1") then
+     return true
+  end
+  return false
+end
+
 local function getInUseExtInterfaces(current_ifid)
   local inuse_ext_interfaces = {}
   local ntopng_interfaces = interface.getIfNames()
   for other_ifid,other_ifname in pairs(ntopng_interfaces) do
     if other_ifid ~= current_ifid then
-      local enabled = ntop.getCache('ntopng.prefs.ifid_'..other_ifid..'.traffic_recording.enabled')
-      if not isEmptyString(enabled) and enabled == "true" then
+      if isRecordingEnabledInCache(other_ifid) then
         local other_ext_ifname = ntop.getCache('ntopng.prefs.ifid_'..other_ifid..'.traffic_recording.ext_ifname')
         if not isEmptyString(other_ext_ifname) then
           inuse_ext_interfaces[other_ext_ifname] = true
@@ -173,17 +240,17 @@ end
 --! @brief Return external interfaces, not in use by ntopng, that can be used through ZMQ interface for traffic recording and flow import
 --! @param ifid the interface identifier 
 --! @return a table with external interfaces information
-function recording_utils.getExtInterfaces(ifid)
+function recording_utils.getExtInterfaces(ifid, show_all)
   local ext_interfaces = {}
   local all_interfaces = ntop.listInterfaces()
   local ntopng_interfaces = swapKeysValues(interface.getIfNames()) 
   local inuse_ext_interfaces = getInUseExtInterfaces(ifid)
- 
+
   for ifname,_ in pairs(all_interfaces) do
-    if ntopng_interfaces[ifname] == nil and -- not in use as packet interface by ntopng 
-       inuse_ext_interfaces[ifname] == nil and -- not in use by other zmq interfaces
-       all_interfaces[ifname].module ~= nil and -- detected by pf_ring
-       all_interfaces[ifname].module ~= "pf_ring" -- ('pf_ring-zc', 'napatech', ..)
+    if ntopng_interfaces[ifname] == nil -- not in use as packet interface by ntopng 
+       and (show_all or inuse_ext_interfaces[ifname] == nil) -- not in use by other zmq interfaces
+       and all_interfaces[ifname].module ~= nil -- detected by pf_ring
+       -- and all_interfaces[ifname].module ~= "pf_ring" -- ('pf_ring-zc', 'napatech', ..)
       then
       local prefix = ""
       if all_interfaces[ifname].module == "pf_ring-zc" then
@@ -253,6 +320,11 @@ function recording_utils.getPcapPath(ifid)
   return storage_path.."/"..ifid.."/pcap"
 end
 
+function recording_utils.getSmartPcapPath(ifid)
+  local storage_path = dirs.pcapdir
+  return storage_path.."/"..ifid.."/smart-pcap"
+end
+
 local function getPcapExtractionPath(ifid)
   local storage_path = dirs.pcapdir
   return storage_path.."/"..ifid.."/extr_pcap"
@@ -263,6 +335,11 @@ local function getTimelinePath(ifid)
   return storage_path.."/"..ifid.."/timeline"
 end
 
+local function getSmartTimelinePath(ifid)
+  local storage_path = dirs.pcapdir
+  return storage_path.."/"..ifid.."/smart-timeline"
+end
+
 local function getPcapFileDir(job_id, ifid)
   local dir_path = getPcapExtractionPath(ifid)
   return dir_path.."/"..job_id
@@ -276,76 +353,85 @@ end
 -- Read information about used disk space for an interface dump
 local function interfaceStorageUsed(ifid)
   local pcap_path = recording_utils.getPcapPath(ifid)
-  if ntop.isdir(pcap_path) then
-    local line = os_utils.execWithOutput("du -s "..pcap_path.." 2>/dev/null")
-    local values = split(line, '\t')
-    if #values >= 1 then
-      local if_used = tonumber(values[1])
-      if if_used ~= nil then
-        if_used = if_used*1024
-        return math.ceil(if_used)
-      end
-    end
-  end
-  return 0
+  return getFolderSize(pcap_path) 
 end
 
---! @brief Read information about the storage, including storage size and available space
---! @param ifid the interface identifier 
---! @return a table containing storage information (size is in bytes)
-function recording_utils.storageInfo(ifid)
-  local storage_info = {
-    path = dirs.pcapdir, dev = "", mount = "",
-    total = 0, used = 0, avail = 0, used_perc = 0,
-    if_used = 0, extraction_used = 0,
+--! @brief Read information about a volume, including storage size and available space
+--! @param path the volume path (or a folder inside the volume)
+function recording_utils.volumeInfo(path)
+  local volume_info = {
+    path = path, 
+    dev = "", 
+    total = 0, 
+    used = 0, 
+    avail = 0, 
+    used_perc = 0,
+    mount = "",
   }
 
-  -- Global storage info
-  local root_path = storage_info.path
+  local root_path = path
   while not ntop.isdir(root_path) and string.len(root_path) > 1 do
     root_path = dirname(root_path) 
   end
+
   if ntop.isdir(root_path) then
-    local line = os_utils.execWithOutput("df "..root_path.." 2>/dev/null|tail -n1")
+    -- use environment variable POSIXLY_CORRECT=1 to guarantee the
+    -- the results are returned in number of 512-byte blocks (otherwise,
+    -- linux will return 1K blocks and BSD 512-byte blocks) 
+    local line = os_utils.execWithOutput("POSIXLY_CORRECT=1 df "..root_path.." 2>/dev/null|tail -n1")
+
     if line ~= nil then
       line = line:gsub('%s+', ' ')
       local values = split(line, ' ')
+
       if #values >= 6 then
-        storage_info.dev = values[1]
-        storage_info.total = tonumber(values[2])*1024
-        storage_info.used = tonumber(values[3])*1024
-        storage_info.avail = tonumber(values[4])*1024
-        storage_info.used_perc = values[5]
-        storage_info.mount = values[6]
+	volume_info.dev = values[1]
+	-- Multiply by 512 as results are in 512-byte blocks 
+        volume_info.total = (tonumber(values[2]) or 0) * 512
+        volume_info.used =  (tonumber(values[3]) or 0) * 512
+        volume_info.avail = (tonumber(values[4]) or 0) * 512
+
+        volume_info.used_perc = values[5]
+        volume_info.mount = values[6]
       end
     end
   end
 
+  return volume_info
+end
+
+--! @brief Read information about the storage, including storage size and available space
+--! @param ifid the interface identifier
+--! @param timeout the maxium time to compute the size
+--! @return a table containing storage information (size is in bytes)
+function recording_utils.storageInfo(ifid, timeout)
+  local storage_info = recording_utils.volumeInfo(dirs.pcapdir)
+
   -- Interface storage info
   storage_info.if_used = interfaceStorageUsed(ifid)
 
   -- PCAP Extraction storage info
   local extraction_path = getPcapExtractionPath(ifid)
-  if ntop.isdir(extraction_path) then
-    local line = os_utils.execWithOutput("du -s "..extraction_path.." 2>/dev/null")
-    local values = split(line, '\t')
-    if #values >= 1 then
-      local extraction_used = tonumber(values[1])
-      if extraction_used ~= nil then
-        storage_info.extraction_used = extraction_used*1024
-      end
-    end
-  end
+  storage_info.extraction_used = getFolderSize(extraction_path, timeout)
 
   return storage_info
 end
 
 local function getN2diskInterfaceName(ifid)
-  if interface.isPacketInterface() then
-    return getInterfaceName(ifid)
-  else
-    return ntop.getCache('ntopng.prefs.ifid_'..ifid..'.traffic_recording.ext_ifname')
-  end
+   local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+
+   if cur_provider == "ntopng" then
+      if interface.isPacketInterface() then
+	 return getInterfaceName(ifid)
+      else
+	 return ntop.getCache('ntopng.prefs.ifid_'..ifid..'.traffic_recording.ext_ifname')
+      end
+   else
+      -- custom provider starts with n2disk@...
+      -- the interface name of a custom provider it is assumed to be the one
+      -- following the initial n2disk@
+      return cur_provider:gsub("^n2disk@", "")
+   end
 end
 
 -- Encode an interface name in a string that can be used in the n2disk configuration file name
@@ -354,6 +440,36 @@ local function getConfigInterfaceName(ifid)
   return ifname:gsub("%,", "_")
 end
 
+function recording_utils.getN2diskInstanceName(ifid)
+  return getConfigInterfaceName(ifid)
+end
+
+local function n2disk_service_name(provider)
+   -- a service that is started manually by the sysadmin is normally called n2disk
+   -- a service that is started and managed by ntopng is called n2disk-ntopng
+   return ternary(provider ~= "ntopng", "n2disk", "n2disk-ntopng")
+end
+
+local function disk2disk_service_name(provider)
+   -- a service that is started manually by the sysadmin is normally called n2disk
+   -- a service that is started and managed by ntopng is called disk2disk-ntopng
+   return ternary(provider ~= "ntopng", "disk2disk", "disk2disk-ntopng")
+end
+
+local function n2diskctl(command, ifid, ...)
+   local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+   local confifname = getConfigInterfaceName(ifid)
+
+   return os_utils.ntopctlCmd(n2disk_service_name(cur_provider), command, confifname, ...)
+end
+
+local function disk2diskctl(command, ifid, ...)
+   local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+   local confifname = getConfigInterfaceName(ifid)
+
+   return os_utils.ntopctlCmd(disk2disk_service_name(cur_provider), command, confifname, ...)
+end
+
 --! @brief Generate a configuration for the traffic recording service (n2disk)
 --! @param ifid the interface identifier 
 --! @param params the traffic recording settings
@@ -372,8 +488,6 @@ function recording_utils.createConfig(ifid, params)
     real_ifname = "zc:"..ifname
   end
 
-  local conf_dir = dirs.workingdir.."/n2disk"
-  local filename = conf_dir.."/n2disk-" .. getConfigInterfaceName(ifid) .. ".conf"
   local storage_path = dirs.pcapdir
 
   if isEmptyString(storage_path) then
@@ -400,8 +514,7 @@ function recording_utils.createConfig(ifid, params)
   local mem_info = memInfo()
   local mem_available_mb = mem_info['MemAvailable']
 
-  -- Computing file and buffer size
-
+  -- Computing file and buffer size based on link speed
   local num_buffered_files = 4
   local min_file_size = 16 
   if ifspeed > 10000 then -- 40/100G
@@ -416,8 +529,8 @@ function recording_utils.createConfig(ifid, params)
     defaults.max_file_size = 64
     num_buffered_files = 2
   end
-  defaults.buffer_size = num_buffered_files * defaults.max_file_size
 
+  defaults.buffer_size = num_buffered_files * defaults.max_file_size
   local total_n2disk_mem = defaults.buffer_size * 2 -- pcap + index buffer
 
   if mem_available_mb ~= nil and mem_available_mb < total_n2disk_mem then
@@ -428,9 +541,14 @@ function recording_utils.createConfig(ifid, params)
       return false
     end
     defaults.buffer_size = (mem_available_mb/2) -- leave some room for index memory and other processes
-    defaults.max_file_size = math.floor(defaults.buffer_size/num_buffered_files)
+    local recomputed_max_file_size = math.floor(defaults.buffer_size/num_buffered_files)
+    if defaults.max_file_size > recomputed_max_file_size then
+      defaults.max_file_size = recomputed_max_file_size
+    end
   end
 
+  -- Checking disk space
+
   -- Computing core affinity
 
   local indexing_threads = 1 -- 1G
@@ -471,8 +589,16 @@ function recording_utils.createConfig(ifid, params)
 
   local config = table.merge(defaults, params)
 
+  -- Recomputing file size based on disk space (2 files + index should fit min)
+  if (config.max_file_size*4) > config.max_disk_space then
+    config.max_file_size = config.max_disk_space/4
+  end
+
   -- Writing configuration file
 
+  local conf_dir = dirs.workingdir.."/n2disk"
+  local filename = conf_dir.."/n2disk-" .. getConfigInterfaceName(ifid) .. ".conf"
+
   local ret = ntop.mkdir(conf_dir)
 
   if not ret then
@@ -497,6 +623,8 @@ function recording_utils.createConfig(ifid, params)
   f:write("--max-file-duration="..config.max_file_duration.."\n")
   f:write("--disk-limit="..config.max_disk_space.."\n")
   f:write("--snaplen="..config.snaplen.."\n")
+
+  -- CPU Core affinity
   f:write("--writer-cpu-affinity="..config.writer_core.."\n")
   f:write("--reader-cpu-affinity="..config.reader_core.."\n")
   f:write("--compressor-cpu-affinity=")
@@ -505,71 +633,426 @@ function recording_utils.createConfig(ifid, params)
   end
   f:write("\n")
   f:write("--index-on-compressor-threads\n")
+
+  -- User
   if not isEmptyString(prefs.user) then
     f:write("-u="..prefs.user.."\n");
   else
     f:write("--dont-change-user\n");
   end
+
+  -- Capture direction
   if interface.isPacketInterface() then
     if prefs.capture_direction == "in" then
       f:write("--capture-direction=1\n")
     elseif prefs.capture_direction == "out" then
       f:write("--capture-direction=2\n")
     else
+      -- Default: RX+TX
       f:write("--capture-direction=0\n")
     end
+  else
+    -- Default on non packet interface: RX+TX
+    f:write("--capture-direction=0\n")
   end
+
+  -- Capture filter (BPF)
+  if not isEmptyString(config.bpf_filter) then
+    f:write("--fast-filter="..config.bpf_filter.."\n")
+  end
+
+  -- ZMQ
   if config.zmq_endpoint ~= nil then
     f:write("--zmq="..config.zmq_endpoint.."\n")
     f:write("--zmq-probe-mode\n")
     f:write("--zmq-export-flows\n")
   end
+
+  -- Index both outer header and tunneled traffic to reflect what ntopng shows
+  f:write("--index-tunnel-content=1\n")
+
   -- Ignored by systemd, required by init.d
   f:write("--daemon\n")
   f:write("-P=/var/run/n2disk-"..ifname..".pid\n")
 
   f:close()
 
+  ---
+
+  if not config.enable_smart_recording then
+    return true
+  end
+
+  -- Writing Smart configuration file
+
+  conf_dir = dirs.workingdir.."/disk2disk"
+  filename = conf_dir.."/disk2disk-" .. getConfigInterfaceName(ifid) .. ".conf"
+
+  local ret = ntop.mkdir(conf_dir)
+
+  if not ret then
+    return false
+  end
+
+  local f = io.open(filename, "w")
+
+  if not f then
+    return false
+  end
+
+  local smart_pcap_path = recording_utils.getSmartPcapPath(ifid)
+  local smart_timeline_path = getSmartTimelinePath(ifid)
+
+  -- Input timeline
+  f:write("-t="..timeline_path.."\n")
+
+  -- Output timeline
+  f:write("-o="..smart_pcap_path.."\n")
+  f:write("-O="..smart_timeline_path.."\n")
+
+  -- Disk settings
+  f:write("-l="..config.max_smart_disk_space.."\n")
+  f:write("-p="..config.max_file_size.."\n")
+
+  -- User
+  if not isEmptyString(prefs.user) then
+    f:write("-u="..prefs.user.."\n");
+  end
+
+  f:close()
+
   return true
 end
 
+local function isRecordingEnabled(ifid)
+  local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+
+  if cur_provider == "ntopng" then
+    return isRecordingEnabledInCache(ifid)
+  else
+    -- if the user has specified a custom provider different than ntopng, it is
+    -- assumed that he/she wants the recording so the service is considered enabled
+    return true
+  end
+end 
+
+local function isSmartRecordingEnabled(ifid)
+  local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+
+  if cur_provider == "ntopng" then
+    return isSmartRecordingEnabledInCache(ifid)
+  else
+    return true
+  end
+end 
+
 --! @brief Check if traffic recording is available and enabled on an interface
 --! @param ifid the interface identifier 
 --! @return true if recording is enabled, false otherwise
 function recording_utils.isEnabled(ifid)
   if recording_utils.isAvailable() then
-    local record_traffic = ntop.getCache('ntopng.prefs.ifid_'..ifid..'.traffic_recording.enabled')
-    if record_traffic == "true" then
-      return true
-    end
+    return isRecordingEnabled(ifid)
   end
+
   return false
 end
 
+function recording_utils.isSmartEnabled(ifid)
+  if not ntop.isEnterpriseXL() then
+    return false
+  end
+
+  if recording_utils.isAvailable() then
+    return isSmartRecordingEnabled(ifid)
+  end
+
+  return false
+end
+
+--! @brief Check if traffic extraction is available and recording is enabled on an interface
+--! @param ifid the interface identifier 
+--! @return true if extraction is available and recording is enabled, false otherwise
+function recording_utils.isExtractionEnabled(ifid)
+  if recording_utils.isExtractionAvailable() then
+    return isRecordingEnabled(ifid)
+  end
+
+  return false
+end
+
+local function isRecordingServiceActive(ifid)
+   local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+   local confifname = getConfigInterfaceName(ifid)
+
+   if cur_provider == "ntopng" then
+      return os_utils.isActive("n2disk-ntopng", confifname)
+   else
+      return os_utils.isActive("n2disk", confifname)
+   end
+end
+
 --! @brief Check if the traffic recording service is running
 --! @param ifid the interface identifier 
 --! @return true if the service is running, false otherwise
 function recording_utils.isActive(ifid)
-  if not recording_utils.isAvailable() then return false end
+   if not recording_utils.isAvailable() then
+      return false
+   end
 
-  local confifname = getConfigInterfaceName(ifid)
-  return os_utils.isActive("n2disk", confifname)
+   return isRecordingServiceActive(ifid)
+end
+
+--! @brief Check if traffic recording is running and extraction is allowed for the current user on an interface
+--! @param ifid the interface identifier 
+--! @return true if the service is running and extraction available, false otherwise
+function recording_utils.isExtractionActive(ifid)
+   if not recording_utils.isExtractionAvailable() then
+      return false
+   end
+
+   return isRecordingServiceActive(ifid)
+end
+
+--! @brief Return statistics from the traffic recording service (n2disk)
+--! @param ifid the interface identifier
+--! @return the statistics
+local function parse_proc_stats(proc_stats)
+   local stats = {}
+
+   local lines = split(proc_stats, "\n") or {}
+
+   for i = 1, #lines do
+      local pair = split(lines[i], ": ")
+      if pair[1] ~= nil and pair[2] ~= nil then
+	 stats[pair[1]] = trimString(pair[2])
+      end
+   end
+
+   if stats['FirstDumpedEpoch'] ~= nil then
+      stats['FirstDumpedEpoch'] = tonumber(stats['FirstDumpedEpoch'])
+   end
+
+   if stats['LastDumpedEpoch'] ~= nil then
+      stats['LastDumpedEpoch'] = tonumber(stats['LastDumpedEpoch'])
+   end
+
+   if stats['Duration'] ~= nil then
+      local u = split(stats['Duration'], ':');
+      stats['DurationEpoch'] = tonumber(u[1])*24*60*60+tonumber(u[2])*60*60+tonumber(u[3])*60+u[4]
+      stats['StartEpoch'] = os.time() - stats['DurationEpoch']
+   end
+
+   return stats
+end
+
+local function is_data_in_window(stats, epoch_begin, epoch_end)
+   local info = {}
+   info.available = false
+
+   local first_epoch = tonumber(stats['FirstDumpedEpoch'])
+   local last_epoch = tonumber(stats['LastDumpedEpoch'])
+
+   if not first_epoch or not last_epoch then
+      traceError(TRACE_WARNING, TRACE_CONSOLE, "Unable to read FirstDumpedEpoch / LastDumpedEpoch from n2disk stats")
+      return info
+   end
+
+   if first_epoch > 0 and last_epoch > 0 and 
+      epoch_end > first_epoch and epoch_begin < last_epoch then
+
+      info.epoch_begin = epoch_begin
+      info.epoch_end = epoch_end
+
+      if first_epoch > epoch_begin then
+         info.epoch_begin = first_epoch
+      end
+      if last_epoch < epoch_end then
+         info.epoch_end = last_epoch
+      end
+
+      info.available = true
+   end
+
+   return info
+end
+
+--! @brief Return statistics from the traffic recording service (n2disk)
+--! @param ifid the interface identifier
+--! @return the statistics
+function recording_utils.stats(ifid)
+   local proc_stats = n2diskctl("stats", ifid)
+   return parse_proc_stats(proc_stats)
+end
+
+--! @brief Return statistics from the traffic recording service (n2disk)
+--! @param ifid the interface identifier
+--! @return the statistics
+function recording_utils.smartStats(ifid)
+   local proc_stats = disk2diskctl("stats", ifid)
+   return parse_proc_stats(proc_stats)
+end
+
+function recording_utils.getAvailableTrafficRecordingProviders()
+   local res = {}
+
+   local list_with_status = os_utils.serviceListWithStatus("n2disk")
+   for _, provider in ipairs(list_with_status) do
+      res[#res + 1] = provider
+   end
+
+   -- this is the default provider, that is, ntopng
+   -- when it manages an n2disk instance for the recording of traffic
+   res[#res + 1] = {name = "ntopng"}
+
+   return res
+end
+
+function recording_utils.getCurrentTrafficRecordingProvider(ifid)
+   local traffic_recording_provider_pref = string.format("ntopng.prefs.traffic_recording.ifid_%d.provider", ifid)
+   local provider = ntop.getCache(traffic_recording_provider_pref)
+
+   return ternary(isEmptyString(provider), "ntopng", provider)
+end
+
+function recording_utils.setCurrentTrafficRecordingProvider(ifid, cur_provider)
+   local traffic_recording_provider_pref = string.format("ntopng.prefs.traffic_recording.ifid_%d.provider", ifid)
+   if cur_provider == "ntopng" then
+      ntop.setCache(traffic_recording_provider_pref, cur_provider)
+      return true
+   end
+
+   local providers = recording_utils.getAvailableTrafficRecordingProviders()
+
+   for _, provider in ipairs(providers) do
+      if provider["name"] == cur_provider then
+	 ntop.setCache(traffic_recording_provider_pref, cur_provider)
+	 return true
+      end
+   end
+
+   return false
+end
+
+--! @brief Returns the path to the timeline of the selected traffic recorder
+--! @param ifid the interface identifier 
+--! @return the timeline, if found, or nil
+function recording_utils.getCurrentTrafficRecordingProviderTimelinePath(ifid)
+   local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+
+   if cur_provider == "ntopng" then
+      return getTimelinePath(ifid)
+   else
+      local stats = recording_utils.stats(ifid)
+      if stats["TimelinePath"] and ntop.exists(stats["TimelinePath"]) then
+	 return stats["TimelinePath"]
+      end
+   end
+end
+
+function recording_utils.getCurrentTrafficRecordingProviderSmartTimelinePath(ifid)
+   local cur_provider = recording_utils.getCurrentTrafficRecordingProvider(ifid)
+
+   if cur_provider == "ntopng" then
+      return getSmartTimelinePath(ifid)
+   else
+      local stats = recording_utils.smartStats(ifid)
+      if stats["TimelinePath"] and ntop.exists(stats["TimelinePath"]) then
+	 return stats["TimelinePath"]
+      end
+   end
+end
+
+--! @brief Return the current n2disk timeline, or the "Smart" timeline if active, based on the time interval
+function recording_utils.getTimelineByInterval(ifid, epoch_begin, epoch_end)
+   if recording_utils.isExtractionEnabled(ifid) then
+      local stats = recording_utils.stats(ifid)
+      local info = is_data_in_window(stats, epoch_begin, epoch_end)
+      if info.available then
+         return recording_utils.getCurrentTrafficRecordingProviderTimelinePath(ifid)
+      end
+   end
+
+   -- Check Smart Recording
+   if recording_utils.isSmartEnabled(ifid) then
+      local stats = recording_utils.smartStats(ifid)
+      local info = is_data_in_window(stats, epoch_begin, epoch_end)
+      if info.available then
+         return recording_utils.getCurrentTrafficRecordingProviderSmartTimelinePath(ifid)
+      end
+   end
+
+   return nil
+end
+
+--! @brief Parse the configuration file of a manually-started n2disk and returns the timeline if found
+--! @param ifid the interface identifier 
+--! @return true if extraction is possible and false otherwise, along with a check message
+function recording_utils.checkExtraction(ifid, use_smart)
+   local res = {}
+   -- try and open the timeline. Failing to open the timeline would cause failing to do the extractions
+
+   local timeline_path
+   if not use_smart then
+      timeline_path = recording_utils.getCurrentTrafficRecordingProviderTimelinePath(ifid)
+   else
+      timeline_path = recording_utils.getCurrentTrafficRecordingProviderSmartTimelinePath(ifid)
+   end
+
+   if not timeline_path then
+      res = {status = "ERR_MISSING_TIMELINE"}
+      -- timeline missing from the conf, won't be able to perform any extraction
+   else
+      if not ntop.exists(timeline_path) then
+	 -- unable to read timeline
+	 res = {status = "ERR_TIMELINE_PATH_NOT_EXISTING"}
+      else
+	 local f = io.open(timeline_path, "r")
+
+	 if not f then
+	    -- unable to open the timeline, insufficient permissions?
+	    res = {status = "ERR_UNABLE_TO_ACCESS_TIMELINE"}
+	 else
+	    -- everything is OK
+	    f:close()
+	    res = {status = "OK"}
+	 end
+      end
+   end
+
+   return res["status"] == "OK", i18n(extraction_checks_to_i18n(res["status"]))
 end
 
 --! @brief Start (or restart) the traffic recording service
 --! @param ifid the interface identifier 
-function recording_utils.restart(ifid)
+function recording_utils.restart(ifid, config)
   local confifname = getConfigInterfaceName(ifid)
-  os_utils.enableService("n2disk", confifname)
-  os_utils.restartService("n2disk", confifname)
+
+  -- Enable Recording
+  os_utils.enableService("n2disk-ntopng", confifname)
+  os_utils.restartService("n2disk-ntopng", confifname)
+
+  -- Enable Smart Recording if requested
+  if (config and config.enable_smart_recording) or
+     (recording_utils.isSmartEnabled(ifid)) then
+    os_utils.enableService("disk2disk-ntopng", confifname) 
+    os_utils.restartService("disk2disk-ntopng", confifname)
+  end
 end
 
 --! @brief Stop the traffic recording service
 --! @param ifid the interface identifier 
 function recording_utils.stop(ifid)
   local confifname = getConfigInterfaceName(ifid)
-  os_utils.stopService("n2disk", confifname)
-  os_utils.disableService("n2disk", confifname)
+
+  -- Stop Recording
+  os_utils.stopService("n2disk-ntopng", confifname)
+  os_utils.disableService("n2disk-ntopng", confifname)
+
+  -- Stop Smart Recording if enabled
+  if recording_utils.isSmartEnabled(ifid) then
+    os_utils.stopService("disk2disk-ntopng", confifname)
+    os_utils.disableService("disk2disk-ntopng", confifname)
+  end
 end
 
 --! @brief Return the log trace of the traffic recording service (n2disk)
@@ -578,28 +1061,7 @@ end
 --| @note lines are retuned in reverse order (most recent line first)
 --! @return the log trace
 function recording_utils.log(ifid, rows)
-  local confifname = getConfigInterfaceName(ifid)
-  local output = n2diskctl("log", confifname, "|tail -n", rows, "|tac")
-  
-  return output
-end
-
---! @brief Return statistics from the traffic recording service (n2disk)
---! @param ifid the interface identifier 
---! @return the statistics
-function recording_utils.stats(ifid)
-  local confifname = getConfigInterfaceName(ifid)
-  local stats = {}
-  local proc_stats = n2diskctl("stats", confifname)
-  local lines = split(proc_stats, "\n")
-
-  for i = 1, #lines do
-    local pair = split(lines[i], ": ")
-    if pair[1] ~= nil and pair[2] ~= nil then
-      stats[pair[1]] = trimString(pair[2])
-    end
-  end
-  return stats
+   return n2diskctl("log", ifid, "|tail -n", rows, "|tac")
 end
 
 -- Read information about used disk space for all ntopng interfaces 
@@ -609,8 +1071,7 @@ local function allInterfacesStorageUsage(ifid)
   local ntopng_interfaces = interface.getIfNames()
   for id,name in pairs(ntopng_interfaces) do
     if ifid == nil or id ~= ifid then
-      local enabled = ntop.getCache('ntopng.prefs.ifid_'..id..'.traffic_recording.enabled')
-      if enabled == "true" then
+      if isRecordingEnabledInCache(id) then
         local disk_space = ntop.getCache('ntopng.prefs.ifid_'..id..'.traffic_recording.disk_space')
         local if_disk_space = 0
         if not isEmptyString(disk_space) then
@@ -639,7 +1100,7 @@ function recording_utils.recommendedSpace(ifid, storage_info)
     avail = 0
   end
 
-  local recommended = avail - (avail*0.2)
+  local recommended = avail - (avail*0.1) -- Leave some room (10%) for index files and other
   return math.floor(recommended)
 end
 
@@ -649,47 +1110,54 @@ end
 --! @param epoch_end the end time (epoch)
 --! @return a table with 'available' = true if the specified interval is included in the dump window, 'epoch_begin'/'epoch_end' are also returned with the actual available window.
 function recording_utils.isDataAvailable(ifid, epoch_begin, epoch_end)
-  local info = {}
-  info.available = false
-  if recording_utils.isEnabled(ifid) then
-    local stats = recording_utils.stats(ifid)
-    if stats['FirstDumpedEpoch'] ~= nil and stats['LastDumpedEpoch'] ~= nil then
-      local first_epoch = tonumber(stats['FirstDumpedEpoch'])
-      local last_epoch = tonumber(stats['LastDumpedEpoch'])
-      if first_epoch > 0 and last_epoch > 0 and 
-         epoch_end > first_epoch and epoch_begin < last_epoch then
-        info.epoch_begin = epoch_begin
-        info.epoch_end = epoch_end
-        if first_epoch > epoch_begin then
-          info.epoch_begin = first_epoch
-        end
-        if last_epoch < epoch_end then
-          info.epoch_end = last_epoch
-        end
-        info.available = true
+   local info = {}
+   info.available = false
+
+   if recording_utils.isExtractionEnabled(ifid) then
+      local stats = recording_utils.stats(ifid)
+      info = is_data_in_window(stats, epoch_begin, epoch_end)
+   end
+
+   if not info.available then
+      -- Check Smart Recording
+      if recording_utils.isSmartEnabled(ifid) then
+         local stats = recording_utils.smartStats(ifid)
+         info = is_data_in_window(stats, epoch_begin, epoch_end)
+         if info.available then
+            info.smart = true
+         end
       end
-    end
-  end
-  return info
+   end
+   
+   if recording_utils.getCurrentTrafficRecordingProvider(ifid) ~= "ntopng" then
+      local extraction_checks_ok, extraction_checks_msg = recording_utils.checkExtraction(ifid, info.smart)
+
+      if not extraction_checks_ok then
+	 info.available = false
+	 info.extraction_checks_msg = extraction_checks_msg
+      end
+   end
+
+   return info
 end
 
 --! @brief Return the list of pcap files extracted for a job
 --! @param job_id the job identifier 
 --! @return the list of pcap files (paths)
 function recording_utils.getJobFiles(id)
-  local job_json = ntop.getHashCache(extraction_jobs_key, id)
-  local files = {}
-  if not isEmptyString(job_json) then
-    local job = json.decode(job_json)
-    local file_id = 1
-    local file = getPcapFilePath(job.id, job.ifid, file_id)
-    while ntop.exists(file) do
-      table.insert(files, file)
-      file_id = file_id + 1
-      file = getPcapFilePath(job.id, job.ifid, file_id) 
-    end
-  end
-  return files
+   local job_json = ntop.getHashCache(extraction_jobs_key, id)
+   local files = {}
+   if not isEmptyString(job_json) then
+      local job = json.decode(job_json)
+      local file_id = 1
+      local file = getPcapFilePath(job.id, job.ifid, file_id)
+      while ntop.exists(file) do
+	 table.insert(files, file)
+	 file_id = file_id + 1
+	 file = getPcapFilePath(job.id, job.ifid, file_id) 
+      end
+   end
+   return files
 end
 
 --! @brief Delete an extraction job and its pcap data on disk, if any
@@ -785,6 +1253,7 @@ function recording_utils.scheduleExtraction(ifid, params)
     time_to = tonumber(params.time_to),
     filter = params.filter,
     chart_url = params.chart_url,
+    timeline_path = params.timeline_path
   }
 
   ntop.setHashCache(extraction_jobs_key, job.id, json.encode(job))
@@ -845,6 +1314,7 @@ function recording_utils.checkExtractionJobs()
       if not isEmptyString(job_json) then
         local job = json.decode(job_json)
 
+        -- job has been stopped, stopping extraction
         ntop.stopExtraction(job.id)
 
         job.status = 'stopped'
@@ -855,12 +1325,14 @@ function recording_utils.checkExtractionJobs()
 
   if not ntop.isExtractionRunning() then
     -- set the previous job as completed, if any
-    setJobAsCompleted()
+    if ntop.getCache(is_running_job_pending_key) == "1" then
+      setJobAsCompleted()
+      ntop.setCache(is_running_job_pending_key, "0")
+    end
 
     -- run a new extraction job
     local id = ntop.lpopCache(extraction_queue_key)
     if not isEmptyString(id) then
-
       local job_json = ntop.getHashCache(extraction_jobs_key, id)
       if not isEmptyString(job_json) then
         local job = json.decode(job_json)
@@ -883,7 +1355,10 @@ function recording_utils.checkExtractionJobs()
             tonumber(job.time_from), 
             tonumber(job.time_to), 
             job.filter, 
-            extraction_limit)
+            extraction_limit,
+	    job.timeline_path)
+
+          ntop.setCache(is_running_job_pending_key, "1")
 
           job.status = 'processing'
           ntop.setHashCache(extraction_jobs_key, job.id, json.encode(job))
@@ -900,5 +1375,9 @@ end
 
 -- #################################
 
+if(trace_script_duration ~= nil) then
+   io.write(debug.getinfo(1,'S').source .." executed in ".. (os.clock()-clock_start)*1000 .. " ms\n")
+end
+
 return recording_utils
 
