@@ -1,5 +1,5 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -25,7 +25,7 @@ local res = {}
 interface.select(getSystemInterfaceId())
 
 -- Add filters
-am_alert_store:add_request_filters()
+am_alert_store:add_request_filters(true)
 
 am_alert_store:delete()
 
