@@ -1,5 +1,5 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -38,7 +38,7 @@ end
 interface.select(ifid)
 
 -- Add filters
-flow_alert_store:add_request_filters()
+flow_alert_store:add_request_filters(true)
 
 flow_alert_store:delete()
 
