@@ -1,5 +1,5 @@
 --
--- (C) 2019-20 - ntop.org
+-- (C) 2019-22 - ntop.org
 --
 dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
@@ -7,9 +7,7 @@ package.path = dirs.installdir .. "/scripts/lua/modules/import_export/?.lua;" ..
 
 require "lua_utils"
 
-local plugins_utils = require("plugins_utils")
-local am_import_export = plugins_utils.loadModule("active_monitoring", "am_import_export")
-local json = require "dkjson"
+local am_import_export = require "am_import_export"
 local rest_utils = require "rest_utils"
 local import_export_rest_utils = require "import_export_rest_utils"
 
@@ -22,7 +20,7 @@ local import_export_rest_utils = require "import_export_rest_utils"
 
 local download = _GET["download"] 
 
-if not haveAdminPrivileges() then
+if not isAdministratorOrPrintErr() then
    rest_utils.answer(rest_utils.consts.err.not_granted)
    return
 end
