@@ -1,5 +1,5 @@
 --
--- (C) 2013-22 - ntop.org
+-- (C) 2013-23 - ntop.org
 --
 
 local dirs = ntop.getDirs()
@@ -30,7 +30,7 @@ end
 interface.select(getSystemInterfaceId())
 
 -- Add filters
-system_alert_store:add_request_filters()
+system_alert_store:add_request_filters(true)
 
 system_alert_store:delete()
 
